@@ -13,27 +13,37 @@ use crate::messaging::data::DataCmd;
 use crate::messaging::{
     data::{CmdError, ServiceMsg},
     system::{KeyedSig, SectionAuth, SystemMsg},
-    DstLocation, MessageId, MessageType, MsgKind, SectionAuthorityProvider, WireMsg,
+    DstLocation, MessageId, MessageType, MsgKind, WireMsg,
 };
 use crate::messaging::{AuthorityProof, ServiceAuth};
+use crate::routing::{log_markers::LogMarker, SectionAuthorityProvider};
+use crate::types::utils::write_data_to_disk;
 use bytes::Bytes;
 use itertools::Itertools;
 use qp2p::ConnectionIncoming;
 use secured_linked_list::SecuredLinkedList;
 use std::net::SocketAddr;
 use tracing::Instrument;
-use xor_name::XorName;
 
 impl Session {
     // Listen for incoming messages on a connection
     #[instrument(skip_all, level = "debug")]
     pub(crate) fn spawn_message_listener_thread(
         session: Session,
+        connection_id: usize,
         src: SocketAddr,
-        mut incoming_messages: ConnectionIncoming<XorName>,
+        mut incoming_messages: ConnectionIncoming,
     ) {
         debug!("Listening for incoming messages");
-        let _ = tokio::spawn(async move {
+
+        trace!(
+            "{} to {} (id: {})",
+            LogMarker::ConnectionOpened,
+            src,
+            connection_id
+        );
+
+        let _handle = tokio::spawn(async move {
             loop {
                 match Self::listen_for_incoming_message(src, &mut incoming_messages).await {
                     Ok(msg) => match Self::handle_msg(msg, src, session.clone()).await {
@@ -52,13 +62,16 @@ impl Session {
                     }
                 }
             }
+
+            // once the message loop breaks, we know the connection is closed
+            trace!("{} to {} (id: {})", LogMarker::ConnectionClosed, src, connection_id);
         }.instrument(info_span!("Listening for incoming msgs"))).in_current_span();
     }
 
     #[instrument(skip_all, level = "debug")]
     pub(crate) async fn listen_for_incoming_message(
         src: SocketAddr,
-        incoming_messages: &mut ConnectionIncoming<XorName>,
+        incoming_messages: &mut ConnectionIncoming,
     ) -> Result<MessageType, Error> {
         if let Some(message) = incoming_messages.next().await? {
             trace!("Incoming message from {:?}", src);
@@ -91,7 +104,7 @@ impl Session {
             } => {
                 let result = Self::handle_ae_redirect_msg(
                     session,
-                    section_auth,
+                    section_auth.into_state(),
                     section_signed,
                     bounced_msg,
                     src,
@@ -114,14 +127,15 @@ impl Session {
             } => {
                 let result = Self::handle_ae_retry_msg(
                     session,
-                    section_auth,
+                    section_auth.into_state(),
                     section_signed,
                     bounced_msg,
                     proof_chain,
+                    src,
                 )
                 .await;
                 if result.is_err() {
-                    warn!("Failed to handle AE-Retry msg");
+                    warn!("Failed to handle AE-Retry msg from {:?}", src);
                 }
                 result
             }
@@ -144,7 +158,7 @@ impl Session {
         let queries = session.pending_queries.clone();
         let error_sender = session.incoming_err_sender;
 
-        let _ = tokio::spawn(async move {
+        let _handle = tokio::spawn(async move {
             match msg {
                 ServiceMsg::QueryResponse { response, .. } => {
                     // Note that this doesn't remove the sender from here since multiple
@@ -183,7 +197,7 @@ impl Session {
                         correlation_id
                     );
                     warn!("CmdError received is: {:?}", error);
-                    let _ = error_sender.send(error.clone()).await;
+                    let _error_from_sender = error_sender.send(error.clone()).await;
 
                     match error {
                         CmdError::Data(_error) => {
@@ -281,6 +295,7 @@ impl Session {
         section_signed: KeyedSig,
         bounced_msg: Bytes,
         proof_chain: SecuredLinkedList,
+        src: SocketAddr,
     ) -> Result<(), Error> {
         // Update our network knowledge making sure proof chain
         // validates the new SAP based on currently known remote section SAP.
@@ -295,20 +310,35 @@ impl Session {
                 if updated {
                     debug!(
                         "Anti-Entropy: updated remote section SAP updated for {:?}",
-                        section_auth.prefix
+                        section_auth.prefix()
                     );
+                    // Update the PrefixMap on disk
+                    if let Err(e) =
+                        write_data_to_disk(&session.network, &session.root_dir.join("prefix_map"))
+                            .await
+                    {
+                        error!(
+                            "Error writing freshly updated PrefixMap to client dir: {:?}",
+                            e
+                        );
+                    }
                 } else {
                     debug!(
                             "Anti-Entropy: discarded SAP for {:?} since it's the same as the one in our records: {:?}",
-                            section_auth.prefix, section_auth
+                            section_auth.prefix(), section_auth
                         );
                 }
             }
             Err(err) => {
                 warn!(
-                        "Anti-Entropy: failed to update remote section SAP, bounced msg dropped. Failed section auth was {:?}, {:?}",
-                        err, section_auth
-                    );
+                    "Anti-Entropy: failed to update remote section SAP w/ err: {:?}",
+                    err
+                );
+                warn!(
+                    "Anti-Entropy: bounced msg dropped. Failed section auth was {:?} sent by: {:?}",
+                    section_auth.section_key(),
+                    src
+                );
                 return Ok(());
             }
         }
@@ -420,32 +450,27 @@ impl Session {
         let target_public_key;
 
         // We normally have received auth when we're in AE-Redirect (where we could not trust enough to update our prefixmap)
-        let mut target_elders = if let Some(auth) = received_auth {
-            target_public_key = auth.public_key_set.public_key();
-            auth.elders
+        let mut target_elders: Vec<_> = if let Some(auth) = received_auth {
+            target_public_key = auth.section_key();
+            auth.elders()
                 .iter()
                 .sorted_by(|(lhs_name, _), (rhs_name, _)| {
                     dst_address_of_bounced_msg.cmp_distance(lhs_name, rhs_name)
                 })
                 .map(|(_, addr)| addr)
                 .take(target_count)
-                .cloned()
-                .collect::<Vec<SocketAddr>>()
+                .copied()
+                .collect()
         } else {
             // we use whatever is our latest knowledge at this point
 
             if let Some(sap) = session
                 .network
-                .closest_or_opposite(&dst_address_of_bounced_msg)
+                .closest_or_opposite(&dst_address_of_bounced_msg, None)
             {
-                target_public_key = sap.value.public_key_set.public_key();
+                target_public_key = sap.section_key();
 
-                sap.value
-                    .elders
-                    .values()
-                    .cloned()
-                    .take(target_count)
-                    .collect::<Vec<SocketAddr>>()
+                sap.elders().values().copied().take(target_count).collect()
             } else {
                 error!("Cannot resend {:?}, no 'received auth' provided, and nothing relevant in session network prefixmap", msg_id);
                 return Ok(None);
@@ -458,13 +483,12 @@ impl Session {
             session.ae_redirect_cache.write().await
         };
 
-        let cache_entry = the_cache_guard.find(|x| {
-            x == &(
-                target_elders.clone(),
-                target_public_key,
-                bounced_msg.clone(),
-            )
-        });
+        let cache_entry =
+            the_cache_guard.find(|(candidate_elders, candidate_public_key, candidate_msg)| {
+                candidate_elders == &target_elders
+                    && candidate_public_key == &target_public_key
+                    && candidate_msg == &bounced_msg
+            });
 
         if cache_entry.is_some() {
             // an elder group corresponds to a PK, so as we've sent to this PK, we've sent to these elders...
