@@ -16,6 +16,7 @@ use crate::messaging::{
     DstLocation, MessageId, MsgKind, ServiceAuth, WireMsg,
 };
 use crate::prefix_map::NetworkPrefixMap;
+use crate::types::utils::write_data_to_disk;
 use crate::types::PublicKey;
 use bytes::Bytes;
 use futures::{future::join_all, stream::FuturesUnordered, TryFutureExt};
@@ -23,6 +24,8 @@ use itertools::Itertools;
 use qp2p::{Config as QuicP2pConfig, Endpoint};
 use rand::rngs::OsRng;
 use rand::seq::SliceRandom;
+use serde::Serialize;
+use std::path::{Path, PathBuf};
 use std::time::Duration;
 use std::{collections::HashMap, net::SocketAddr, sync::Arc};
 use tokio::{
@@ -39,9 +42,12 @@ pub(crate) const NUM_OF_ELDERS_SUBSET_FOR_QUERIES: usize = 3;
 // Number of bootstrap nodes to attempt to contact per batch (if provided by the node_config)
 pub(crate) const NODES_TO_CONTACT_PER_STARTUP_BATCH: usize = 3;
 
+// Root directory for Clients
+pub(crate) const SAFE_CLIENT_DIR: &str = ".safe/client";
+
 impl Session {
     /// Acquire a session by bootstrapping to a section, maintaining connections to several nodes.
-    #[instrument(skip_all, level = "debug")]
+    #[instrument(skip(err_sender), level = "debug")]
     pub(crate) async fn new(
         client_pk: PublicKey,
         genesis_key: bls::PublicKey,
@@ -51,14 +57,16 @@ impl Session {
         standard_wait: Duration,
         prefix_map: NetworkPrefixMap,
     ) -> Result<Session, Error> {
-        trace!(
-            "Trying to bootstrap to the network with public_key: {:?}",
-            client_pk
-        );
-        debug!("QP2p config: {:?}", qp2p_config);
+        trace!("Trying to bootstrap to the network");
 
         let endpoint = Endpoint::new_client(local_addr, qp2p_config)?;
 
+        // Create client's root dir
+        let root_dir = create_client_root_dir(client_pk).await?;
+
+        // Write our PrefixMap to disk in our root dir
+        write_data_to_path(&prefix_map, &root_dir.join("prefix_map")).await?;
+
         let session = Session {
             client_pk,
             pending_queries: Arc::new(RwLock::new(HashMap::default())),
@@ -71,6 +79,7 @@ impl Session {
             genesis_key,
             initial_connection_check_msg_id: Arc::new(RwLock::new(None)),
             standard_wait,
+            root_dir,
         };
 
         Ok(session)
@@ -89,18 +98,18 @@ impl Session {
         // TODO: Consider other approach: Keep a session per section!
 
         // Get DataSection elders details.
-        let (elders, section_pk) = if let Some(sap) = self.network.closest_or_opposite(&dst_address)
-        {
-            let sap_elders = sap.value.elders.values().cloned();
+        let (elders, section_pk) =
+            if let Some(sap) = self.network.closest_or_opposite(&dst_address, None) {
+                let sap_elders = sap.elders().values().copied();
 
-            trace!("{:?} SAP elders found", sap_elders);
-            (
-                sap_elders.take(targets_count).collect::<Vec<SocketAddr>>(),
-                sap.value.public_key_set.public_key(),
-            )
-        } else {
-            return Err(Error::NoNetworkKnowledge);
-        };
+                trace!("{:?} SAP elders found", sap_elders);
+                (
+                    sap_elders.take(targets_count).collect::<Vec<SocketAddr>>(),
+                    sap.section_key(),
+                )
+            } else {
+                return Err(Error::NoNetworkKnowledge);
+            };
 
         let msg_id = MessageId::new();
 
@@ -164,19 +173,19 @@ impl Session {
         let dst = query.dst_name();
 
         // Get DataSection elders details. Resort to own section if DataSection is not available.
-        let (elders, section_pk) = if let Some(sap) = self.network.closest_or_opposite(&dst) {
-            (sap.value.elders, sap.value.public_key_set.public_key())
+        let (section_pk, elders) = if let Some(sap) = self.network.closest_or_opposite(&dst, None) {
+            (sap.section_key(), sap.value.elders().clone())
         } else {
             return Err(Error::NoNetworkKnowledge);
         };
 
         // We select the NUM_OF_ELDERS_SUBSET_FOR_QUERIES closest Elders we are querying
-        let chosen_elders = elders
+        let chosen_elders: Vec<_> = elders
             .into_iter()
             .sorted_by(|(lhs_name, _), (rhs_name, _)| dst.cmp_distance(lhs_name, rhs_name))
             .map(|(_, addr)| addr)
             .take(NUM_OF_ELDERS_SUBSET_FOR_QUERIES)
-            .collect::<Vec<SocketAddr>>();
+            .collect();
 
         let elders_len = chosen_elders.len();
         if elders_len < NUM_OF_ELDERS_SUBSET_FOR_QUERIES && elders_len > 1 {
@@ -203,7 +212,7 @@ impl Session {
 
         let pending_queries_for_thread = pending_queries.clone();
         if let Ok(op_id) = query.operation_id() {
-            let _ = tokio::spawn(async move {
+            let _handle = tokio::spawn(async move {
                 // Insert the response sender
                 trace!("Inserting channel for op_id {:?}", op_id);
                 let _old = pending_queries_for_thread
@@ -242,13 +251,12 @@ impl Session {
                         .connect_to(&socket)
                         .err_into()
                         .and_then(|(connection, connection_incoming)| async move {
-                            if let Some(connection_incoming) = connection_incoming {
-                                Self::spawn_message_listener_thread(
-                                    session,
-                                    connection.remote_address(),
-                                    connection_incoming,
-                                );
-                            }
+                            Self::spawn_message_listener_thread(
+                                session,
+                                connection.id(),
+                                connection.remote_address(),
+                                connection_incoming,
+                            );
                             connection.send_with(msg_bytes, priority, None).await
                         })
                         .await;
@@ -342,7 +350,7 @@ impl Session {
 
         if let Some(query) = &response {
             if let Ok(query_op_id) = query.operation_id() {
-                let _ = tokio::spawn(async move {
+                let _handle = tokio::spawn(async move {
                     // Remove the response sender
                     trace!("Removing channel for {:?}", query_op_id);
                     let _old_channel = pending_queries.clone().write().await.remove(&query_op_id);
@@ -364,13 +372,6 @@ impl Session {
         }
     }
 
-    // /// This tells us if we've seen at least _one_ AE-Retry msg (and attempted to resend it + cached that)
-    // /// This can be useful to know if we're still working with _only_ genesis key knowledge
-    // #[instrument(skip_all, level = "debug")]
-    // pub(crate) async fn has_seen_ae_retry(&self) -> bool {
-
-    // }
-
     #[instrument(skip_all, level = "debug")]
     pub(crate) async fn make_contact_with_nodes(
         &self,
@@ -383,18 +384,17 @@ impl Session {
         // Get DataSection elders details.
         // TODO: we should be able to handle using an pre-existing prefixmap. This is here for when that's in place.
         let (elders_or_adults, section_pk) =
-            if let Some(sap) = self.network.closest_or_opposite(&dst_address) {
-                let mut nodes = sap
-                    .value
-                    .elders
+            if let Some(sap) = self.network.closest_or_opposite(&dst_address, None) {
+                let mut nodes: Vec<_> = sap
+                    .elders()
                     .values()
-                    .cloned()
                     .take(NUM_OF_ELDERS_SUBSET_FOR_QUERIES)
-                    .collect::<Vec<SocketAddr>>();
+                    .copied()
+                    .collect();
 
                 nodes.shuffle(&mut OsRng);
 
-                (nodes, sap.value.public_key_set.public_key())
+                (nodes, sap.section_key())
             } else {
                 // Send message to our bootstrap peer with network's genesis PK.
                 (nodes, self.genesis_key)
@@ -417,7 +417,11 @@ impl Session {
         let msg_kind = MsgKind::ServiceMsg(auth);
         let wire_msg = WireMsg::new_msg(msg_id, payload, msg_kind, dst_location)?;
 
-        let initial_contacts = elders_or_adults[0..NODES_TO_CONTACT_PER_STARTUP_BATCH].to_vec();
+        let initial_contacts = elders_or_adults
+            .iter()
+            .take(NODES_TO_CONTACT_PER_STARTUP_BATCH)
+            .copied()
+            .collect();
         send_message(
             self.clone(),
             initial_contacts,
@@ -439,7 +443,11 @@ impl Session {
         // If we start with genesis key here, we should wait until we have _at least_ one AE-Retry in
         if section_pk == self.genesis_key {
             // wait until we have _some_ network knowledge
-            while self.network.closest_or_opposite(&dst_address).is_none() {
+            while self
+                .network
+                .closest_or_opposite(&dst_address, None)
+                .is_none()
+            {
                 let stats = self.network.known_sections_count();
                 debug!("Client still has not received any AE-Retry message... {:?}. Current sections known", stats);
 
@@ -474,6 +482,10 @@ impl Session {
                     )
                     .await?;
 
+                    trace!(
+                        "Awaiting a duration of {:?} before trying new nodes",
+                        self.standard_wait
+                    );
                     tokio::time::sleep(self.standard_wait).await;
                 }
             }
@@ -481,14 +493,6 @@ impl Session {
 
         Ok(())
     }
-
-    #[allow(unused)]
-    pub(crate) async fn disconnect_from_peers(&self, peers: Vec<SocketAddr>) -> Result<(), Error> {
-        for elder in peers {
-            self.endpoint.disconnect_from(&elder).await;
-        }
-        Ok(())
-    }
 }
 
 #[instrument(skip_all, level = "trace")]
@@ -496,7 +500,7 @@ pub(super) async fn send_message(
     session: Session,
     elders: Vec<SocketAddr>,
     wire_msg: WireMsg,
-    endpoint: Endpoint<XorName>,
+    endpoint: Endpoint,
     msg_id: MessageId,
 ) -> Result<(), Error> {
     let priority = wire_msg.msg_kind().priority();
@@ -520,13 +524,12 @@ pub(super) async fn send_message(
                     .connect_to(&socket)
                     .err_into()
                     .and_then(|(connection, connection_incoming)| async move {
-                        if let Some(connection_incoming) = connection_incoming {
-                            Session::spawn_message_listener_thread(
-                                session,
-                                connection.remote_address(),
-                                connection_incoming,
-                            );
-                        }
+                        Session::spawn_message_listener_thread(
+                            session,
+                            connection.id(),
+                            connection.remote_address(),
+                            connection_incoming,
+                        );
                         connection.send_with(msg_bytes_clone, priority, None).await
                     })
                     .await?;
@@ -573,3 +576,28 @@ pub(super) async fn send_message(
 
     Ok(())
 }
+
+#[instrument(skip_all, level = "trace")]
+pub(crate) async fn write_data_to_path<T: Serialize>(data: &T, path: &Path) -> Result<(), Error> {
+    // Write our PrefixMap to root dir
+    if let Err(e) = write_data_to_disk(data, path).await {
+        error!("Error writing data for Client at dir {:?}: {:?}", path, e);
+    }
+
+    Ok(())
+}
+
+#[instrument(skip_all, level = "trace")]
+pub(crate) async fn create_client_root_dir(client_pk: PublicKey) -> Result<PathBuf, Error> {
+    let mut root_dir = dirs_next::home_dir()
+        .ok_or_else(|| Error::Generic("Error opening home dir".to_string()))?;
+    root_dir.push(SAFE_CLIENT_DIR);
+    root_dir.push(format!("sn_client-{}", client_pk));
+
+    // Create `.safe/client` dir if not present
+    tokio::fs::create_dir_all(root_dir.clone())
+        .await
+        .map_err(|e| Error::Generic(format!("Error creating client root dir: {:?}", e)))?;
+
+    Ok(root_dir)
+}
