@@ -7,7 +7,7 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::{agreement::SectionAuth, section::NodeState};
-use crate::messaging::SectionAuthorityProvider;
+use crate::messaging::{system::KeyedSig, SectionAuthorityProvider};
 use bls::PublicKey as BlsPublicKey;
 use ed25519_dalek::Signature;
 use secured_linked_list::SecuredLinkedList;
@@ -55,7 +55,16 @@ pub enum JoinResponse {
         nonce_signature: Signature,
     },
     /// Up to date section information for a joining peer to retry its join request with
-    Retry(SectionAuthorityProvider),
+    Retry {
+        /// Current `SectionAuthorityProvider` of the section.
+        section_auth: SectionAuthorityProvider,
+        /// Section signature over the `SectionAuthorityProvider`.
+        section_signed: KeyedSig,
+        /// Section chain truncated from the section key found in the join request.
+        proof_chain: SecuredLinkedList,
+        /// The age of the node as expected by the section.
+        expected_age: u8,
+    },
     /// Response redirecting a joining peer to join a different section,
     /// containing addresses of nodes that are closer (than the recipient) to the
     /// requested name. The `JoinRequest` should be re-sent to these addresses.
