@@ -6,28 +6,20 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{
-    delivery_group, split_barrier::SplitBarrier, Comm, Core, SignatureAggregator, KEY_CACHE_SIZE,
-    RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY,
-};
+use super::{delivery_group, Comm, Core};
 use crate::dbs::UsedSpace;
-use crate::messaging::{
-    system::{NodeState, Peer, Proposal, Section},
-    SectionAuthorityProvider, WireMsg,
-};
-use crate::prefix_map::NetworkPrefixMap;
+use crate::messaging::{system::Proposal, WireMsg};
 use crate::routing::{
-    dkg::{DkgVoter, ProposalAggregator},
     error::Result,
+    log_markers::LogMarker,
+    network_knowledge::{NetworkKnowledge, NodeState, SectionAuthorityProvider, SectionKeyShare},
     node::Node,
     routing_api::command::Command,
-    section::{NodeStateUtils, SectionKeyShare, SectionKeysProvider},
-    Event,
+    Event, Peer,
 };
-use resource_proof::ResourceProof;
 use secured_linked_list::SecuredLinkedList;
-use std::{collections::BTreeSet, net::SocketAddr, path::PathBuf, sync::Arc};
-use tokio::sync::{mpsc, RwLock};
+use std::{collections::BTreeSet, net::SocketAddr, path::PathBuf};
+use tokio::sync::mpsc;
 use xor_name::XorName;
 
 impl Core {
@@ -38,11 +30,13 @@ impl Core {
         event_tx: mpsc::Sender<Event>,
         used_space: UsedSpace,
         root_storage_dir: PathBuf,
+        genesis_sk_set: bls::SecretKeySet,
     ) -> Result<Self> {
         // make sure the Node has the correct local addr as Comm
         node.addr = comm.our_connection_info();
 
-        let (section, section_key_share) = Section::first_node(node.peer())?;
+        let (section, section_key_share) =
+            NetworkKnowledge::first_node(node.peer(), genesis_sk_set).await?;
         Self::new(
             comm,
             node,
@@ -56,62 +50,40 @@ impl Core {
         .await
     }
 
-    pub(crate) async fn relocated(&self, mut new_node: Node, new_section: Section) -> Result<Self> {
-        let section_keys_provider = SectionKeysProvider::new(KEY_CACHE_SIZE, None).await;
+    pub(crate) async fn relocate(
+        &self,
+        mut new_node: Node,
+        new_section: NetworkKnowledge,
+    ) -> Result<()> {
+        // we first try to relocate section info.
+        self.network_knowledge.relocated_to(new_section).await?;
 
         // make sure the new Node has the correct local addr as Comm
-        let comm = self.comm.clone();
-        new_node.addr = comm.our_connection_info();
+        new_node.addr = self.comm.our_connection_info();
 
-        let network = NetworkPrefixMap::new(*new_section.genesis_key());
-        // TODO: to keep our knowledge of the network and avoid unnecessary AE msgs:
-        // - clone self.network instead,
-        // - remove the SAP of our new section from the cloned network
-        // - and add current section's SAP to the cloned network
+        let mut our_node = self.node.write().await;
+        *our_node = new_node;
 
-        Ok(Self {
-            comm,
-            node: new_node,
-            section: new_section,
-            network,
-            section_keys_provider,
-            proposal_aggregator: ProposalAggregator::default(),
-            split_barrier: Arc::new(RwLock::new(SplitBarrier::new())),
-            message_aggregator: SignatureAggregator::default(),
-            dkg_voter: DkgVoter::default(),
-            relocate_state: None,
-            event_tx: self.event_tx.clone(),
-            joins_allowed: Arc::new(RwLock::new(true)),
-            is_genesis_node: false,
-            resource_proof: ResourceProof::new(RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY),
-            register_storage: self.register_storage.clone(),
-            root_storage_dir: self.root_storage_dir.clone(),
-            used_space: self.used_space.clone(),
-            capacity: self.capacity.clone(),
-            chunk_storage: self.chunk_storage.clone(),
-            liveness: self.liveness.clone(),
-        })
+        Ok(())
     }
 
-    pub(crate) fn node(&self) -> &Node {
-        &self.node
+    pub(crate) fn network_knowledge(&self) -> &NetworkKnowledge {
+        &self.network_knowledge
     }
 
-    pub(crate) fn section(&self) -> &Section {
-        &self.section
-    }
-
-    pub(crate) fn section_chain(&self) -> &SecuredLinkedList {
-        self.section.chain()
+    pub(crate) async fn section_chain(&self) -> SecuredLinkedList {
+        self.network_knowledge.chain().await
     }
 
     /// Is this node an elder?
-    pub(crate) fn is_elder(&self) -> bool {
-        self.section.is_elder(&self.node.name())
+    pub(crate) async fn is_elder(&self) -> bool {
+        self.network_knowledge
+            .is_elder(&self.node.read().await.name())
+            .await
     }
 
-    pub(crate) fn is_not_elder(&self) -> bool {
-        !self.is_elder()
+    pub(crate) async fn is_not_elder(&self) -> bool {
+        !self.is_elder().await
     }
 
     /// Returns connection info of this node.
@@ -130,28 +102,32 @@ impl Core {
 
     /// Returns the current BLS public key set
     pub(crate) async fn public_key_set(&self) -> Result<bls::PublicKeySet> {
-        Ok(self.section_keys_provider.key_share().await?.public_key_set)
+        Ok(self.key_share().await?.public_key_set)
     }
 
     /// Returns the info about the section matching the name.
-    pub(crate) fn matching_section(&self, name: &XorName) -> Result<SectionAuthorityProvider> {
-        if self.section.prefix().matches(name) {
-            Ok(self.section.authority_provider().clone())
+    pub(crate) async fn matching_section(
+        &self,
+        name: &XorName,
+    ) -> Result<SectionAuthorityProvider> {
+        if self.network_knowledge.prefix().await.matches(name) {
+            Ok(self.network_knowledge.authority_provider().await)
         } else {
-            self.network.section_by_name(name)
+            self.network_knowledge.section_by_name(name)
         }
     }
 
     /// Returns our index in the current BLS group if this node is a member of one, or
     /// `Error::MissingSecretKeyShare` otherwise.
     pub(crate) async fn our_index(&self) -> Result<usize> {
-        Ok(self.section_keys_provider.key_share().await?.index)
+        Ok(self.key_share().await?.index)
     }
 
     /// Returns our key share in the current BLS group if this node is a member of one, or
     /// `Error::MissingSecretKeyShare` otherwise.
     pub(crate) async fn key_share(&self) -> Result<SectionKeyShare> {
-        self.section_keys_provider.key_share().await
+        let section_key = self.network_knowledge.section_key().await;
+        self.section_keys_provider.key_share(&section_key).await
     }
 
     pub(crate) async fn send_event(&self, event: Event) {
@@ -165,20 +141,23 @@ impl Core {
     //   ---------------------------------- Mut ------------------------------------------
     // ----------------------------------------------------------------------------------------
 
-    pub(crate) fn handle_timeout(&mut self, token: u64) -> Result<Vec<Command>> {
-        self.dkg_voter
-            .handle_timeout(&self.node, token, *self.section_chain().last_key())
+    pub(crate) async fn handle_timeout(&self, token: u64) -> Result<Vec<Command>> {
+        self.dkg_voter.handle_timeout(
+            &self.node.read().await.clone(),
+            token,
+            self.network_knowledge().section_key().await,
+        )
     }
 
     // Send message to peers on the network.
-    pub(crate) fn send_msg_to_peers(&self, mut wire_msg: WireMsg) -> Result<Command> {
+    pub(crate) async fn send_msg_to_peers(&self, mut wire_msg: WireMsg) -> Result<Command> {
         let dst_location = wire_msg.dst_location();
         let (targets, dg_size) = delivery_group::delivery_targets(
             dst_location,
-            &self.node.name(),
-            &self.section,
-            &self.network,
-        )?;
+            &self.node.read().await.name(),
+            &self.network_knowledge,
+        )
+        .await?;
 
         trace!(
             "relay {:?} to first {:?} of {:?} (Section PK: {:?})",
@@ -192,10 +171,10 @@ impl Core {
 
         // To avoid loop: if destination is to Node, targets are multiple, self is an elder,
         //     self section prefix matches the destination name, then don't carry out a relay.
-        if self.is_elder()
+        if self.is_elder().await
             && targets.len() > 1
             && dst_location.is_to_node()
-            && self.section.prefix().matches(&target_name)
+            && self.network_knowledge.prefix().await.matches(&target_name)
         {
             return Ok(Command::SendMessageDeliveryGroup {
                 recipients: Vec::new(),
@@ -204,14 +183,11 @@ impl Core {
             });
         }
 
-        let dst_pk = self.section_key_by_name(&target_name);
+        let dst_pk = self.section_key_by_name(&target_name).await;
         wire_msg.set_dst_section_pk(dst_pk);
 
         let command = Command::SendMessageDeliveryGroup {
-            recipients: targets
-                .into_iter()
-                .map(|peer| (peer.name, peer.addr))
-                .collect(),
+            recipients: targets.into_iter().collect(),
             delivery_group_size: dg_size,
             wire_msg,
         };
@@ -222,7 +198,7 @@ impl Core {
     // Setting the JoinsAllowed triggers a round Proposal::SetJoinsAllowed to update the flag.
     pub(crate) async fn set_joins_allowed(&self, joins_allowed: bool) -> Result<Vec<Command>> {
         let mut commands = Vec::new();
-        if self.is_elder() && joins_allowed != *self.joins_allowed.read().await {
+        if self.is_elder().await && joins_allowed != *self.joins_allowed.read().await {
             commands.extend(self.propose(Proposal::JoinsAllowed(joins_allowed)).await?);
         }
         Ok(commands)
@@ -230,7 +206,7 @@ impl Core {
 
     // Generate a new section info based on the current set of members and if it differs from the
     // current elders, trigger a DKG.
-    pub(crate) async fn promote_and_demote_elders(&mut self) -> Result<Vec<Command>> {
+    pub(crate) async fn promote_and_demote_elders(&self) -> Result<Vec<Command>> {
         self.promote_and_demote_elders_except(&BTreeSet::new())
             .await
     }
@@ -244,9 +220,11 @@ impl Core {
     ) -> Result<Vec<Command>> {
         let mut commands = vec![];
 
+        debug!("{}", LogMarker::TriggeringPromotionAndDemotion);
         for elder_candidates in self
-            .section
-            .promote_and_demote_elders(&self.node.name(), excluded_names)
+            .network_knowledge
+            .promote_and_demote_elders(&self.node.read().await.name(), excluded_names)
+            .await
         {
             commands.extend(self.send_dkg_start(elder_candidates).await?);
         }
@@ -256,12 +234,12 @@ impl Core {
 
     pub(crate) async fn make_online_proposal(
         &self,
-        peer: Peer,
+        peer: &Peer,
         previous_name: Option<XorName>,
         dst_key: Option<bls::PublicKey>,
     ) -> Result<Vec<Command>> {
         self.propose(Proposal::Online {
-            node_state: NodeState::joined(peer, previous_name),
+            node_state: NodeState::joined(peer, previous_name).into_msg(),
             dst_key,
         })
         .await
