@@ -6,31 +6,34 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
+use super::{read_prefix_map_from_disk, UsedRecipientSaps};
 use crate::messaging::{
     system::{
-        JoinRejectionReason, JoinRequest, JoinResponse, ResourceProofResponse, Section, SystemMsg,
+        JoinRejectionReason, JoinRequest, JoinResponse, ResourceProofResponse, SectionAuth,
+        SystemMsg,
     },
     DstLocation, MessageType, MsgKind, NodeAuth, WireMsg,
 };
+use crate::prefix_map::NetworkPrefixMap;
 use crate::routing::{
     core::{Comm, ConnectionEvent, SendStatus},
     dkg::SectionAuthUtils,
     ed25519,
     error::{Error, Result},
+    log_markers::LogMarker,
     messages::{NodeMsgAuthorityUtils, WireMsgUtils},
+    network_knowledge::NetworkKnowledge,
     node::Node,
-    peer::PeerUtils,
-    SectionAuthorityProviderUtils, FIRST_SECTION_MAX_AGE, FIRST_SECTION_MIN_AGE, MIN_ADULT_AGE,
+    Peer,
 };
-
+use backoff::{backoff::Backoff, ExponentialBackoff};
 use bls::PublicKey as BlsPublicKey;
 use futures::future;
-use rand::seq::IteratorRandom;
 use resource_proof::ResourceProof;
-use std::{collections::HashSet, net::SocketAddr};
-use tokio::sync::mpsc;
+use std::net::SocketAddr;
+use tokio::{sync::mpsc, time::Duration};
 use tracing::Instrument;
-use xor_name::{Prefix, XorName};
+use xor_name::Prefix;
 
 /// Join the network as new node.
 ///
@@ -43,42 +46,52 @@ pub(crate) async fn join_network(
     incoming_conns: &mut mpsc::Receiver<ConnectionEvent>,
     bootstrap_addr: SocketAddr,
     genesis_key: BlsPublicKey,
-) -> Result<(Node, Section)> {
+) -> Result<(Node, NetworkKnowledge)> {
     let (send_tx, send_rx) = mpsc::channel(1);
 
     let span = trace_span!("bootstrap");
 
-    let state = Join::new(node, send_tx, incoming_conns);
+    // Read prefix map from cache if available
+    let prefix_map = read_prefix_map_from_disk(genesis_key).await?;
 
-    future::join(
-        state.run(bootstrap_addr, genesis_key),
-        send_messages(send_rx, comm),
-    )
-    .instrument(span)
-    .await
-    .0
+    let state = Join::new(node, send_tx, incoming_conns, prefix_map);
+
+    future::join(state.run(bootstrap_addr), send_messages(send_rx, comm))
+        .instrument(span)
+        .await
+        .0
 }
 
 struct Join<'a> {
     // Sender for outgoing messages.
-    send_tx: mpsc::Sender<(WireMsg, Vec<(XorName, SocketAddr)>)>,
+    send_tx: mpsc::Sender<(WireMsg, Vec<Peer>)>,
     // Receiver for incoming messages.
     recv_rx: &'a mut mpsc::Receiver<ConnectionEvent>,
     node: Node,
     prefix: Prefix,
+    prefix_map: NetworkPrefixMap,
+    backoff: ExponentialBackoff,
 }
 
 impl<'a> Join<'a> {
     fn new(
         node: Node,
-        send_tx: mpsc::Sender<(WireMsg, Vec<(XorName, SocketAddr)>)>,
+        send_tx: mpsc::Sender<(WireMsg, Vec<Peer>)>,
         recv_rx: &'a mut mpsc::Receiver<ConnectionEvent>,
+        prefix_map: NetworkPrefixMap,
     ) -> Self {
         Self {
             send_tx,
             recv_rx,
             node,
             prefix: Prefix::default(),
+            prefix_map,
+            backoff: ExponentialBackoff {
+                initial_interval: Duration::from_millis(50),
+                max_interval: Duration::from_millis(750),
+                max_elapsed_time: Some(Duration::from_secs(60)),
+                ..Default::default()
+            },
         }
     }
 
@@ -88,23 +101,21 @@ impl<'a> Join<'a> {
     // - `ResourceChallenge`: carry out resource proof calculation.
     // - `Approval`: returns the initial `Section` value to use by this node,
     //    completing the bootstrap.
-    async fn run(
-        self,
-        bootstrap_addr: SocketAddr,
-        genesis_key: BlsPublicKey,
-    ) -> Result<(Node, Section)> {
+    async fn run(self, bootstrap_addr: SocketAddr) -> Result<(Node, NetworkKnowledge)> {
         // Use our XorName as we do not know their name or section key yet.
         let dst_xorname = self.node.name();
-        let recipients = vec![(dst_xorname, bootstrap_addr)];
+        let recipients = vec![Peer::new(dst_xorname, bootstrap_addr)];
+        let genesis_key = self.prefix_map.genesis_key();
 
         self.join(genesis_key, recipients).await
     }
 
+    #[tracing::instrument(skip(self))]
     async fn join(
         mut self,
         network_genesis_key: BlsPublicKey,
-        mut recipients: Vec<(XorName, SocketAddr)>,
-    ) -> Result<(Node, Section)> {
+        recipients: Vec<Peer>,
+    ) -> Result<(Node, NetworkKnowledge)> {
         // We first use genesis key as the target section key, we'll be getting
         // a response with the latest section key for us to retry with.
         // Once we are approved to join, we will make sure the SAP we receive can
@@ -119,16 +130,14 @@ impl<'a> Join<'a> {
             resource_proof_response: None,
         };
 
-        self.send_join_requests(join_request, &recipients, section_key)
+        self.send_join_requests(join_request, &recipients, section_key, false)
             .await?;
 
-        // Avoid sending more than one request to the same peer.
-        let mut used_recipient = HashSet::<SocketAddr>::new();
+        // Avoid sending more than one duplicated request (with same SectionKey) to the same peer.
+        let mut used_recipient_saps = UsedRecipientSaps::new();
 
         loop {
-            used_recipient.extend(recipients.iter().map(|(_, addr)| addr));
-
-            let (response, sender, src_name) = self.receive_join_response().await?;
+            let (response, sender) = self.receive_join_response().await?;
 
             match response {
                 JoinResponse::Rejected(JoinRejectionReason::NodeNotReachable(addr)) => {
@@ -156,24 +165,26 @@ impl<'a> Join<'a> {
                         continue;
                     }
 
+                    trace!("{}", LogMarker::ReceivedJoinApproved);
+
                     return Ok((
                         self.node,
-                        Section::new(genesis_key, section_chain, section_auth)?,
+                        NetworkKnowledge::new(
+                            genesis_key,
+                            section_chain,
+                            section_auth.into_authed_state(),
+                            Some(self.prefix_map),
+                        )?,
                     ));
                 }
-                JoinResponse::Retry(section_auth) => {
-                    let new_recipients: Vec<(XorName, SocketAddr)> = section_auth
-                        .elders
-                        .iter()
-                        .map(|(name, addr)| (*name, *addr))
-                        .collect();
+                JoinResponse::Retry {
+                    section_auth,
+                    section_signed,
+                    proof_chain,
+                    expected_age,
+                } => {
+                    let section_auth = section_auth.into_state();
 
-                    let prefix = section_auth.prefix;
-
-                    // For the first section, using age random among 6 to 100 to avoid
-                    // relocating too many nodes at the same time.
-                    // To avoid recursive due to received outdated SAP, self prefix shall be default
-                    // as well when re-generate for a new age.
                     trace!(
                         "Joining node {:?} - {:?}/{:?} received a Retry with SAP {:?}",
                         self.prefix,
@@ -181,108 +192,119 @@ impl<'a> Join<'a> {
                         self.node.age(),
                         section_auth
                     );
-                    if prefix.is_empty()
-                        && self.prefix.is_empty()
-                        && self.node.age() < FIRST_SECTION_MIN_AGE
-                    {
-                        let age: u8 = (FIRST_SECTION_MIN_AGE..FIRST_SECTION_MAX_AGE)
-                            .choose(&mut rand::thread_rng())
-                            .unwrap_or(FIRST_SECTION_MAX_AGE);
 
-                        let new_keypair =
-                            ed25519::gen_keypair(&Prefix::default().range_inclusive(), age);
+                    let prefix = section_auth.prefix();
+                    if !prefix.matches(&self.node.name()) {
+                        warn!(
+                            "Ignoring newer JoinResponse::Retry response not for us {:?}, SAP {:?} from {:?}",
+                            self.node.name(),
+                            section_auth,
+                            sender,
+                        );
+                        continue;
+                    }
+
+                    let signed_sap = SectionAuth {
+                        value: section_auth.clone(),
+                        sig: section_signed,
+                    };
+
+                    // make sure we received a valid and trusted new SAP
+                    match self.prefix_map.update(signed_sap, &proof_chain) {
+                        Ok(true) => {}
+                        Ok(false) => {
+                            // it's not a new SAP, ignore it unless the expected age is different.
+                            if self.node.age() == expected_age {
+                                debug!("Ignoring JoinResponse::Retry with same SAP as we previously sent to: {:?}", section_auth);
+                                continue;
+                            }
+                        }
+                        Err(err) => {
+                            debug!(
+                                "Ignoring JoinResponse::Retry with an invalid SAP: {:?}",
+                                err
+                            );
+                            continue;
+                        }
+                    }
+
+                    // make sure our joining age is the expected by the network
+                    if self.node.age() != expected_age {
+                        trace!("re-generate name due to mis-matched age, current {:?} vs. expected {:?}",
+                            self.node.age(), expected_age);
+                        let new_keypair = ed25519::gen_keypair(
+                            &Prefix::default().range_inclusive(),
+                            expected_age,
+                        );
                         let new_name = ed25519::name(&new_keypair.public);
 
-                        info!("Setting Node name to {}", new_name);
+                        info!("Setting Node name to {} (age {})", new_name, expected_age);
                         self.node = Node::new(new_keypair, self.node.addr);
                     }
 
-                    if prefix.matches(&self.node.name()) {
-                        self.prefix = prefix;
-                        // After section split, new node must join with the age of MIN_ADULT_AGE.
-                        if !prefix.is_empty() && self.node.age() != MIN_ADULT_AGE {
-                            let new_keypair =
-                                ed25519::gen_keypair(&prefix.range_inclusive(), MIN_ADULT_AGE);
-                            let new_name = ed25519::name(&new_keypair.public);
+                    info!(
+                        "Newer Join response for us {:?}, SAP {:?} from {:?}",
+                        self.node.name(),
+                        section_auth,
+                        sender
+                    );
 
-                            info!("Setting Node name to {}", new_name);
-                            self.node = Node::new(new_keypair, self.node.addr);
-                        }
+                    section_key = section_auth.section_key();
+                    let join_request = JoinRequest {
+                        section_key,
+                        resource_proof_response: None,
+                    };
 
-                        info!(
-                            "Newer Join response for us {:?}, SAP {:?} from {:?}",
-                            self.node.name(),
-                            section_auth,
-                            sender
-                        );
-                        section_key = section_auth.section_key();
-                        let join_request = JoinRequest {
-                            section_key,
-                            resource_proof_response: None,
-                        };
+                    let new_recipients = section_auth.peers();
+                    self.send_join_requests(join_request, &new_recipients, section_key, true)
+                        .await?;
+                }
+                JoinResponse::Redirect(section_auth) => {
+                    let section_auth = section_auth.into_state();
 
-                        recipients = new_recipients;
-                        self.send_join_requests(join_request, &recipients, section_key)
-                            .await?;
-                    } else {
+                    if !section_auth.prefix().matches(&self.node.name()) {
                         warn!(
-                            "Newer Join response not for us {:?}, SAP {:?} from {:?}",
+                            "Ignoring newer JoinResponse::Redirect response not for us {:?}, SAP {:?} from {:?}",
                             self.node.name(),
                             section_auth,
                             sender,
                         );
-                    }
-                }
-                JoinResponse::Redirect(section_auth) => {
-                    if section_auth.section_key() == section_key {
-                        debug!("Ignoring JoinResponse::Redirect with same section authority provider key as we previously sent: {:?}", section_auth);
                         continue;
                     }
 
-                    // Ignore already used recipients
-                    let new_recipients: Vec<(XorName, SocketAddr)> = section_auth
-                        .elders
-                        .iter()
-                        .filter(|(_, addr)| !used_recipient.contains(addr))
-                        .map(|(name, addr)| (*name, *addr))
+                    let new_section_key = section_auth.section_key();
+
+                    let new_recipients: Vec<_> = section_auth
+                        .peers()
+                        .into_iter()
+                        .filter(|peer| used_recipient_saps.insert((peer.addr(), new_section_key)))
                         .collect();
 
                     if new_recipients.is_empty() {
-                        debug!("Joining redirected to the same set of peers we already contacted - ignoring response");
+                        debug!(
+                            "Ignoring JoinResponse::Redirect with old SAP that has been sent to: {:?}",
+                            section_auth
+                        );
                         continue;
-                    } else {
-                        info!(
-                            "Joining redirected to another set of peers: {:?}",
-                            new_recipients,
-                        );
                     }
 
-                    if section_auth.prefix.matches(&self.node.name()) {
-                        self.prefix = section_auth.prefix;
-                        info!(
-                            "Newer Join response for us {:?}, SAP {:?} from {:?}",
-                            self.node.name(),
-                            section_auth,
-                            sender
-                        );
-                        section_key = section_auth.section_key();
-                        let join_request = JoinRequest {
-                            section_key,
-                            resource_proof_response: None,
-                        };
+                    info!(
+                        "Newer JoinResponse::Redirect for us {:?}, SAP {:?} from {:?}",
+                        self.node.name(),
+                        section_auth,
+                        sender
+                    );
 
-                        recipients = new_recipients;
+                    section_key = new_section_key;
+                    self.prefix = section_auth.prefix();
 
-                        self.send_join_requests(join_request, &recipients, section_key)
-                            .await?;
-                    } else {
-                        warn!(
-                            "Newer Join response not for us {:?}, SAP {:?} from {:?}",
-                            self.node.name(),
-                            section_auth,
-                            sender,
-                        );
-                    }
+                    let join_request = JoinRequest {
+                        section_key,
+                        resource_proof_response: None,
+                    };
+
+                    self.send_join_requests(join_request, &new_recipients, section_key, true)
+                        .await?;
                 }
                 JoinResponse::ResourceChallenge {
                     data_size,
@@ -304,20 +326,36 @@ impl<'a> Join<'a> {
                             nonce_signature,
                         }),
                     };
-                    let recipients = &[(src_name, sender)];
-                    self.send_join_requests(join_request, recipients, section_key)
+                    let recipients = &[sender];
+                    self.send_join_requests(join_request, recipients, section_key, false)
                         .await?;
                 }
             }
         }
     }
 
+    #[tracing::instrument(skip(self))]
     async fn send_join_requests(
         &mut self,
         join_request: JoinRequest,
-        recipients: &[(XorName, SocketAddr)],
+        recipients: &[Peer],
         section_key: BlsPublicKey,
+        should_backoff: bool,
     ) -> Result<()> {
+        if should_backoff {
+            // use exponential backoff here to delay our responses and avoid any intensive join reqs
+            let next_wait = self.backoff.next_backoff();
+
+            if let Some(wait) = next_wait {
+                tokio::time::sleep(wait).await;
+            } else {
+                error!("Waiting before attempting to join again");
+
+                tokio::time::sleep(2 * self.backoff.max_interval).await;
+                self.backoff.reset();
+            }
+        }
+
         info!("Sending {:?} to {:?}", join_request, recipients);
 
         let node_msg = SystemMsg::JoinRequest(Box::new(join_request));
@@ -331,24 +369,25 @@ impl<'a> Join<'a> {
             section_key,
         )?;
 
-        let _ = self.send_tx.send((wire_msg, recipients.to_vec())).await;
+        let _res = self.send_tx.send((wire_msg, recipients.to_vec())).await;
 
         Ok(())
     }
 
     // TODO: receive JoinResponse from the JoinResponse handler directly,
     // analogous to the JoinAsRelocated flow.
-    async fn receive_join_response(&mut self) -> Result<(JoinResponse, SocketAddr, XorName)> {
+    #[tracing::instrument(skip(self))]
+    async fn receive_join_response(&mut self) -> Result<(JoinResponse, Peer)> {
         while let Some(event) = self.recv_rx.recv().await {
             // we are interested only in `JoinResponse` type of messages
-            let (join_response, sender, src_name) = match event {
-                ConnectionEvent::Received((sender, bytes)) => match WireMsg::from(bytes) {
+            let (join_response, sender) = match event {
+                ConnectionEvent::Received((sender_addr, bytes)) => match WireMsg::from(bytes) {
                     Ok(wire_msg) => match wire_msg.msg_kind() {
                         MsgKind::ServiceMsg(_) => continue,
                         MsgKind::NodeBlsShareAuthMsg(_) => {
                             trace!(
                                 "Bootstrap message discarded: sender: {:?} wire_msg: {:?}",
-                                sender,
+                                sender_addr,
                                 wire_msg
                             );
                             continue;
@@ -358,14 +397,17 @@ impl<'a> Join<'a> {
                                 msg: SystemMsg::JoinResponse(resp),
                                 msg_authority,
                                 ..
-                            }) => (*resp, sender, msg_authority.src_location().name()),
+                            }) => (
+                                *resp,
+                                Peer::new(msg_authority.src_location().name(), sender_addr),
+                            ),
                             Ok(
                                 MessageType::Service { msg_id, .. }
                                 | MessageType::System { msg_id, .. },
                             ) => {
                                 trace!(
                                     "Bootstrap message discarded: sender: {:?} msg_id: {:?}",
-                                    sender,
+                                    sender_addr,
                                     msg_id
                                 );
                                 continue;
@@ -387,9 +429,11 @@ impl<'a> Join<'a> {
                 JoinResponse::ResourceChallenge { .. }
                 | JoinResponse::Rejected(JoinRejectionReason::NodeNotReachable(_))
                 | JoinResponse::Rejected(JoinRejectionReason::JoinsDisallowed) => {
-                    return Ok((join_response, sender, src_name));
+                    return Ok((join_response, sender));
+                }
+                JoinResponse::Retry {
+                    ref section_auth, ..
                 }
-                JoinResponse::Retry(ref section_auth)
                 | JoinResponse::Redirect(ref section_auth) => {
                     if section_auth.elders.is_empty() {
                         error!(
@@ -398,9 +442,9 @@ impl<'a> Join<'a> {
                         );
                         continue;
                     }
-                    trace!("Received a redirect/retry JoinResponse. Sending request to the latest contacts");
+                    trace!("Received a redirect/retry JoinResponse from {}. Sending request to the latest contacts", sender);
 
-                    return Ok((join_response, sender, src_name));
+                    return Ok((join_response, sender));
                 }
                 JoinResponse::Approval {
                     ref section_auth,
@@ -408,7 +452,7 @@ impl<'a> Join<'a> {
                     ref section_chain,
                     ..
                 } => {
-                    if node_state.value.peer.name() != &self.node.name() {
+                    if node_state.name != self.node.name() {
                         trace!("Ignore NodeApproval not for us");
                         continue;
                     }
@@ -428,10 +472,10 @@ impl<'a> Join<'a> {
 
                     trace!(
                         "This node has been approved to join the network at {:?}!",
-                        section_auth.value.prefix,
+                        section_auth.prefix,
                     );
 
-                    return Ok((join_response, sender, src_name));
+                    return Ok((join_response, sender));
                 }
             }
         }
@@ -443,10 +487,7 @@ impl<'a> Join<'a> {
 }
 
 // Keep reading messages from `rx` and send them using `comm`.
-async fn send_messages(
-    mut rx: mpsc::Receiver<(WireMsg, Vec<(XorName, SocketAddr)>)>,
-    comm: &Comm,
-) -> Result<()> {
+async fn send_messages(mut rx: mpsc::Receiver<(WireMsg, Vec<Peer>)>, comm: &Comm) -> Result<()> {
     while let Some((wire_msg, recipients)) = rx.recv().await {
         match comm
             .send(&recipients, recipients.len(), wire_msg.clone())
@@ -470,11 +511,13 @@ async fn send_messages(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::messaging::{system::NodeState, SectionAuthorityProvider};
+    use crate::messaging::SectionAuthorityProvider as SectionAuthorityProviderMsg;
     use crate::routing::{
-        dkg::test_utils::*, error::Error as RoutingError, messages::WireMsgUtils,
-        section::test_utils::*, section::NodeStateUtils, SectionAuthorityProviderUtils, ELDER_SIZE,
-        MIN_ADULT_AGE, MIN_AGE,
+        dkg::test_utils::*,
+        error::Error as RoutingError,
+        messages::WireMsgUtils,
+        network_knowledge::{test_utils::*, NodeState},
+        ELDER_SIZE, MIN_ADULT_AGE,
     };
     use crate::types::PublicKey;
     use assert_matches::assert_matches;
@@ -486,6 +529,7 @@ mod tests {
     use secured_linked_list::SecuredLinkedList;
     use std::collections::BTreeMap;
     use tokio::task;
+    use xor_name::XorName;
 
     #[tokio::test(flavor = "multi_thread")]
     async fn join_as_adult() -> Result<()> {
@@ -496,22 +540,26 @@ mod tests {
             gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
         let bootstrap_node = nodes.remove(0);
         let bootstrap_addr = bootstrap_node.addr;
-
         let sk = sk_set.secret_key();
-        let pk = sk.public_key();
+        let section_key = sk.public_key();
 
-        // Node in first section has to have an age higher than MIN_ADULT_AGE
+        // Node in first section has to have a stepped age,
         // Otherwise during the bootstrap process, node will change its id and age.
-        let node_age = MIN_AGE + 2;
+        let node_age = MIN_ADULT_AGE;
         let node = Node::new(
             ed25519::gen_keypair(&Prefix::default().range_inclusive(), node_age),
             gen_addr(),
         );
         let peer = node.peer();
-        let state = Join::new(node, send_tx, &mut recv_rx);
+        let state = Join::new(
+            node,
+            send_tx,
+            &mut recv_rx,
+            NetworkPrefixMap::new(section_key),
+        );
 
         // Create the bootstrap task, but don't run it yet.
-        let bootstrap = async move { state.run(bootstrap_addr, pk).await.map_err(Error::from) };
+        let bootstrap = async move { state.run(bootstrap_addr).await.map_err(Error::from) };
 
         // Create the task that executes the body of the test, but don't run it either.
         let others = async {
@@ -521,8 +569,10 @@ mod tests {
                 .await
                 .ok_or_else(|| eyre!("JoinRequest was not received"))?;
 
-            let bootstrap_addrs: Vec<SocketAddr> =
-                recipients.iter().map(|(_name, addr)| *addr).collect();
+            let bootstrap_addrs: Vec<SocketAddr> = recipients
+                .iter()
+                .map(|recipient| recipient.addr())
+                .collect();
             assert_eq!(bootstrap_addrs, [bootstrap_addr]);
 
             let node_msg = assert_matches!(wire_msg.into_message(), Ok(MessageType::System { msg, .. }) =>
@@ -533,9 +583,17 @@ mod tests {
             });
 
             // Send JoinResponse::Retry with section auth provider info
+            let section_chain = SecuredLinkedList::new(section_key);
+            let signed_sap = section_signed(sk, section_auth.clone())?;
+
             send_response(
                 &recv_tx,
-                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(section_auth.clone()))),
+                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
+                    section_auth: section_auth.clone().into_msg(),
+                    section_signed: signed_sap.sig,
+                    proof_chain: section_chain,
+                    expected_age: MIN_ADULT_AGE,
+                })),
                 &bootstrap_node,
                 section_auth.section_key(),
             )?;
@@ -548,33 +606,26 @@ mod tests {
             let (node_msg, dst_location) = assert_matches!(wire_msg.into_message(), Ok(MessageType::System { msg, dst_location,.. }) =>
                 (msg, dst_location));
 
-            assert_eq!(dst_location.section_pk(), Some(pk));
-            itertools::assert_equal(
-                recipients,
-                section_auth
-                    .elders()
-                    .iter()
-                    .map(|(name, addr)| (*name, *addr))
-                    .collect::<Vec<_>>(),
-            );
+            assert_eq!(dst_location.section_pk(), Some(section_key));
+            itertools::assert_equal(recipients, section_auth.peers());
             assert_matches!(node_msg, SystemMsg::JoinRequest(request) => {
-                assert_eq!(request.section_key, pk);
+                assert_eq!(request.section_key, section_key);
             });
 
             // Send JoinResponse::Approval
             let section_auth = section_signed(sk, section_auth.clone())?;
-            let node_state = section_signed(sk, NodeState::joined(peer, None))?;
-            let proof_chain = SecuredLinkedList::new(pk);
+            let node_state = section_signed(sk, NodeState::joined(&peer, None))?;
+            let proof_chain = SecuredLinkedList::new(section_key);
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Approval {
-                    genesis_key: pk,
-                    section_auth: section_auth.clone(),
-                    node_state,
+                    genesis_key: section_key,
+                    section_auth: section_auth.clone().into_authed_msg(),
+                    node_state: node_state.into_authed_msg(),
                     section_chain: proof_chain,
                 })),
                 &bootstrap_node,
-                section_auth.value.section_key(),
+                section_auth.section_key(),
             )?;
 
             Ok(())
@@ -583,8 +634,8 @@ mod tests {
         // Drive both tasks to completion concurrently (but on the same thread).
         let ((node, section), _) = future::try_join(bootstrap, others).await?;
 
-        assert_eq!(*section.authority_provider(), section_auth);
-        assert_eq!(*section.chain().last_key(), pk);
+        assert_eq!(section.authority_provider().await, section_auth);
+        assert_eq!(section.section_key().await, section_key);
         assert_eq!(node.age(), node_age);
 
         Ok(())
@@ -603,9 +654,14 @@ mod tests {
             ed25519::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
             gen_addr(),
         );
-        let state = Join::new(node, send_tx, &mut recv_rx);
+        let state = Join::new(
+            node,
+            send_tx,
+            &mut recv_rx,
+            NetworkPrefixMap::new(genesis_key),
+        );
 
-        let bootstrap_task = state.run(bootstrap_node.addr, genesis_key);
+        let bootstrap_task = state.run(bootstrap_node.addr);
         let test_task = async move {
             // Receive JoinRequest
             let (wire_msg, recipients) = send_rx
@@ -616,7 +672,7 @@ mod tests {
             assert_eq!(
                 recipients
                     .into_iter()
-                    .map(|peer| peer.1)
+                    .map(|peer| peer.addr())
                     .collect::<Vec<_>>(),
                 vec![bootstrap_node.addr]
             );
@@ -636,7 +692,7 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(
-                    SectionAuthorityProvider {
+                    SectionAuthorityProviderMsg {
                         prefix: Prefix::default(),
                         public_key_set: new_pk_set.clone(),
                         elders: new_bootstrap_addrs.clone(),
@@ -656,7 +712,7 @@ mod tests {
             assert_eq!(
                 recipients
                     .into_iter()
-                    .map(|peer| peer.1)
+                    .map(|peer| peer.addr())
                     .collect::<Vec<_>>(),
                 new_bootstrap_addrs
                     .iter()
@@ -686,6 +742,9 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn join_invalid_redirect_response() -> Result<()> {
+        crate::init_test_logger();
+        let _span = tracing::info_span!("join_invalid_redirect_response").entered();
+
         let (send_tx, mut send_rx) = mpsc::channel(1);
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
@@ -696,9 +755,15 @@ mod tests {
             ed25519::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
             gen_addr(),
         );
-        let state = Join::new(node, send_tx, &mut recv_rx);
+        let section_key = sk_set.secret_key().public_key();
+        let state = Join::new(
+            node,
+            send_tx,
+            &mut recv_rx,
+            NetworkPrefixMap::new(section_key),
+        );
 
-        let bootstrap_task = state.run(bootstrap_node.addr, sk_set.secret_key().public_key());
+        let bootstrap_task = state.run(bootstrap_node.addr);
         let test_task = async {
             let (wire_msg, _) = send_rx
                 .recv()
@@ -706,7 +771,7 @@ mod tests {
                 .ok_or_else(|| eyre!("JoinRequest was not received"))?;
 
             assert_matches!(wire_msg.into_message(), Ok(MessageType::System { msg, .. }) =>
-                        assert_matches!(msg, SystemMsg::JoinRequest{..}));
+            assert_matches!(msg, SystemMsg::JoinRequest{..}));
 
             let (new_section_auth, _, new_sk_set) =
                 gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
@@ -715,7 +780,7 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(
-                    SectionAuthorityProvider {
+                    SectionAuthorityProviderMsg {
                         prefix: Prefix::default(),
                         public_key_set: new_pk_set.clone(),
                         elders: BTreeMap::new(),
@@ -733,7 +798,7 @@ mod tests {
             send_response(
                 &recv_tx,
                 SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(
-                    SectionAuthorityProvider {
+                    SectionAuthorityProviderMsg {
                         prefix: Prefix::default(),
                         public_key_set: new_pk_set.clone(),
                         elders: addrs,
@@ -750,7 +815,7 @@ mod tests {
                 .ok_or_else(|| eyre!("JoinRequest was not received"))?;
 
             assert_matches!(wire_msg.into_message(), Ok(MessageType::System { msg, .. }) =>
-                            assert_matches!(msg, SystemMsg::JoinRequest{..}));
+            assert_matches!(msg, SystemMsg::JoinRequest{..}));
 
             Ok(())
         };
@@ -778,9 +843,15 @@ mod tests {
             gen_addr(),
         );
 
-        let state = Join::new(node, send_tx, &mut recv_rx);
+        let section_key = sk_set.secret_key().public_key();
+        let state = Join::new(
+            node,
+            send_tx,
+            &mut recv_rx,
+            NetworkPrefixMap::new(section_key),
+        );
 
-        let bootstrap_task = state.run(bootstrap_node.addr, sk_set.secret_key().public_key());
+        let bootstrap_task = state.run(bootstrap_node.addr);
         let test_task = async {
             let (wire_msg, _) = send_rx
                 .recv()
@@ -814,6 +885,9 @@ mod tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn join_invalid_retry_prefix_response() -> Result<()> {
+        crate::init_test_logger();
+        let _span = tracing::info_span!("join_invalid_retry_prefix_response").entered();
+
         let (send_tx, mut send_rx) = mpsc::channel(1);
         let (recv_tx, mut recv_rx) = mpsc::channel(1);
 
@@ -838,11 +912,18 @@ mod tests {
             }
         };
 
-        let state = Join::new(node, send_tx, &mut recv_rx);
+        let (section_auth, _, sk_set) = gen_section_authority_provider(good_prefix, ELDER_SIZE);
+        let section_key = sk_set.public_keys().public_key();
+
+        let state = Join::new(
+            node,
+            send_tx,
+            &mut recv_rx,
+            NetworkPrefixMap::new(section_key),
+        );
 
-        let section_key = bls::SecretKey::random().public_key();
         let elders = (0..ELDER_SIZE)
-            .map(|_| (good_prefix.substituted_in(rand::random()), gen_addr()))
+            .map(|_| Peer::new(good_prefix.substituted_in(rand::random()), gen_addr()))
             .collect();
         let join_task = state.join(section_key, elders);
 
@@ -856,12 +937,20 @@ mod tests {
                 assert_matches!(wire_msg.into_message(), Ok(MessageType::System{ msg, .. }) => msg);
             assert_matches!(node_msg, SystemMsg::JoinRequest(_));
 
+            let section_chain = SecuredLinkedList::new(section_key);
+            let signed_sap = section_signed(sk_set.secret_key(), section_auth.clone())?;
+
             // Send `Retry` with bad prefix
             send_response(
                 &recv_tx,
-                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(
-                    gen_section_authority_provider(bad_prefix, ELDER_SIZE).0,
-                ))),
+                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
+                    section_auth: gen_section_authority_provider(bad_prefix, ELDER_SIZE)
+                        .0
+                        .into_msg(),
+                    section_signed: signed_sap.sig.clone(),
+                    proof_chain: section_chain.clone(),
+                    expected_age: MIN_ADULT_AGE,
+                })),
                 &bootstrap_node,
                 section_key,
             )?;
@@ -870,9 +959,12 @@ mod tests {
             // Send `Retry` with good prefix
             send_response(
                 &recv_tx,
-                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(
-                    gen_section_authority_provider(good_prefix, ELDER_SIZE).0,
-                ))),
+                SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
+                    section_auth: section_auth.into_msg(),
+                    section_signed: signed_sap.sig,
+                    proof_chain: section_chain,
+                    expected_age: MIN_ADULT_AGE,
+                })),
                 &bootstrap_node,
                 section_key,
             )?;
@@ -899,6 +991,7 @@ mod tests {
     }
 
     // test helper
+    #[instrument]
     fn send_response(
         recv_tx: &mpsc::Sender<ConnectionEvent>,
         node_msg: SystemMsg,
@@ -915,6 +1008,8 @@ mod tests {
             section_pk,
         )?;
 
+        debug!("wire msg built");
+
         recv_tx.try_send(ConnectionEvent::Received((
             bootstrap_node.addr,
             wire_msg.serialize()?,
