@@ -6,28 +6,28 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
+use super::UsedRecipientSaps;
 use crate::messaging::{
     system::{
-        JoinAsRelocatedRequest, JoinAsRelocatedResponse, RelocateDetails, RelocatePayload, Section,
+        JoinAsRelocatedRequest, JoinAsRelocatedResponse, RelocateDetails, RelocatePayload,
         SystemMsg,
     },
-    AuthorityProof, DstLocation, SectionAuth, SectionAuthorityProvider, WireMsg,
+    AuthorityProof, DstLocation, SectionAuth, WireMsg,
 };
 use crate::routing::{
     dkg::SectionAuthUtils,
     ed25519,
     error::{Error, Result},
-    log_markers::LogMarker,
     messages::WireMsgUtils,
+    network_knowledge::{NetworkKnowledge, SectionAuthorityProvider},
     node::Node,
-    peer::PeerUtils,
     relocation::RelocatePayloadUtils,
     routing_api::command::Command,
-    SectionAuthorityProviderUtils,
+    Peer,
 };
 use crate::types::PublicKey;
 use bls::PublicKey as BlsPublicKey;
-use std::{collections::HashSet, net::SocketAddr};
+use std::net::SocketAddr;
 use xor_name::{Prefix, XorName};
 
 /// Re-join as a relocated node.
@@ -38,8 +38,8 @@ pub(crate) struct JoiningAsRelocated {
     relocate_details: RelocateDetails,
     node_msg: SystemMsg,
     node_msg_auth: AuthorityProof<SectionAuth>,
-    // Avoid sending more than one request to the same peer.
-    used_recipients: HashSet<SocketAddr>,
+    // Avoid sending more than one duplicated request (with same SectionKey) to the same peer.
+    used_recipient_saps: UsedRecipientSaps,
     relocate_payload: Option<RelocatePayload>,
 }
 
@@ -60,7 +60,7 @@ impl JoiningAsRelocated {
             relocate_details,
             node_msg,
             node_msg_auth: section_auth,
-            used_recipients: HashSet::<SocketAddr>::new(),
+            used_recipient_saps: UsedRecipientSaps::new(),
             relocate_payload: None,
         })
     }
@@ -69,12 +69,15 @@ impl JoiningAsRelocated {
     // shall be fed back with `handle_join_response` function.
     pub(crate) fn start(&mut self, bootstrap_addrs: Vec<SocketAddr>) -> Result<Command> {
         let dst_xorname = self.relocate_details.dst;
-        let recipients: Vec<(XorName, SocketAddr)> = bootstrap_addrs
+        let recipients: Vec<_> = bootstrap_addrs
             .iter()
-            .map(|addr| (dst_xorname, *addr))
+            .map(|addr| Peer::new(dst_xorname, *addr))
             .collect();
 
-        self.used_recipients.extend(bootstrap_addrs);
+        self.used_recipient_saps = bootstrap_addrs
+            .iter()
+            .map(|addr| (*addr, self.dst_section_key))
+            .collect();
 
         // We send a first join request to obtain the section prefix, which
         // we will then use to generate the relocation payload and send the
@@ -99,7 +102,7 @@ impl JoiningAsRelocated {
                 section_chain,
                 node_state,
             } => {
-                if node_state.value.peer.name() != &self.node.name() {
+                if node_state.name != self.node.name() {
                     trace!("Ignore NodeApproval not for us");
                     return Ok(None);
                 }
@@ -120,15 +123,22 @@ impl JoiningAsRelocated {
 
                 trace!(
                     "This node has been approved to join the network at {:?}!",
-                    section_auth.value.prefix,
+                    section_auth.prefix,
                 );
 
                 Ok(Some(Command::HandleRelocationComplete {
                     node: self.node.clone(),
-                    section: Section::new(self.genesis_key, section_chain, section_auth)?,
+                    // FIXME: pass our existing prefixmap here
+                    section: NetworkKnowledge::new(
+                        self.genesis_key,
+                        section_chain,
+                        section_auth.into_authed_state(),
+                        None,
+                    )?,
                 }))
             }
             JoinAsRelocatedResponse::Retry(section_auth) => {
+                let section_auth = section_auth.into_state();
                 if !self.check_autority_provider(&section_auth, &self.relocate_details.dst) {
                     return Ok(None);
                 }
@@ -137,16 +147,28 @@ impl JoiningAsRelocated {
                     return Ok(None);
                 }
 
-                let new_recipients: Vec<(XorName, SocketAddr)> = section_auth
-                    .elders
-                    .iter()
-                    .map(|(name, addr)| (*name, *addr))
+                let new_section_key = section_auth.section_key();
+                let new_recipients: Vec<_> = section_auth
+                    .peers()
+                    .into_iter()
+                    .filter(|peer| {
+                        self.used_recipient_saps
+                            .insert((peer.addr(), new_section_key))
+                    })
                     .collect();
 
+                if new_recipients.is_empty() {
+                    debug!(
+                        "Ignore JoinAsRelocatedResponse::Retry with old SAP that has been sent to: {:?}",
+                        section_auth
+                    );
+                    return Ok(None);
+                }
+
                 // if we are relocating, and we didn't generate
                 // the relocation payload yet, we do it now
                 if self.relocate_payload.is_none() {
-                    self.build_relocation_payload(&section_auth.prefix)?;
+                    self.build_relocation_payload(&section_auth.prefix())?;
                 }
 
                 info!(
@@ -156,12 +178,12 @@ impl JoiningAsRelocated {
                 self.dst_section_key = section_auth.section_key();
 
                 let cmd = self.build_join_request_cmd(&new_recipients)?;
-                self.used_recipients
-                    .extend(new_recipients.iter().map(|(_, addr)| addr));
 
                 Ok(Some(cmd))
             }
             JoinAsRelocatedResponse::Redirect(section_auth) => {
+                let section_auth = section_auth.into_state();
+
                 if !self.check_autority_provider(&section_auth, &self.relocate_details.dst) {
                     return Ok(None);
                 }
@@ -170,28 +192,28 @@ impl JoiningAsRelocated {
                     return Ok(None);
                 }
 
-                // Ignore already used recipients
-                let new_recipients: Vec<(XorName, SocketAddr)> = section_auth
-                    .elders
-                    .iter()
-                    .filter(|(_, addr)| !self.used_recipients.contains(addr))
-                    .map(|(name, addr)| (*name, *addr))
+                let new_section_key = section_auth.section_key();
+                let new_recipients: Vec<_> = section_auth
+                    .peers()
+                    .into_iter()
+                    .filter(|peer| {
+                        self.used_recipient_saps
+                            .insert((peer.addr(), new_section_key))
+                    })
                     .collect();
 
                 if new_recipients.is_empty() {
-                    debug!("Joining redirected to the same set of peers we already contacted - ignoring response");
-                    return Ok(None);
-                } else {
-                    info!(
-                        "Joining redirected to another set of peers: {:?}",
-                        new_recipients,
+                    debug!(
+                        "Ignore JoinAsRelocatedResponse::Redirect with old SAP that has been sent to: {:?}",
+                        section_auth
                     );
+                    return Ok(None);
                 }
 
                 // if we are relocating, and we didn't generate
                 // the relocation payload yet, we do it now
                 if self.relocate_payload.is_none() {
-                    self.build_relocation_payload(&section_auth.prefix)?;
+                    self.build_relocation_payload(&section_auth.prefix())?;
                 }
 
                 info!(
@@ -201,8 +223,6 @@ impl JoiningAsRelocated {
                 self.dst_section_key = section_auth.section_key();
 
                 let cmd = self.build_join_request_cmd(&new_recipients)?;
-                self.used_recipients
-                    .extend(new_recipients.iter().map(|(_, addr)| addr));
 
                 Ok(Some(cmd))
             }
@@ -242,7 +262,7 @@ impl JoiningAsRelocated {
         Ok(())
     }
 
-    fn build_join_request_cmd(&self, recipients: &[(XorName, SocketAddr)]) -> Result<Command> {
+    fn build_join_request_cmd(&self, recipients: &[Peer]) -> Result<Command> {
         let join_request = JoinAsRelocatedRequest {
             section_key: self.dst_section_key,
             relocate_payload: self.relocate_payload.clone(),
@@ -261,8 +281,6 @@ impl JoiningAsRelocated {
             self.genesis_key,
         )?;
 
-        trace!("{}", LogMarker::SendJoinRequest);
-
         let cmd = Command::SendMessage {
             recipients: recipients.to_vec(),
             wire_msg,
@@ -276,10 +294,10 @@ impl JoiningAsRelocated {
         section_auth: &SectionAuthorityProvider,
         dst: &XorName,
     ) -> bool {
-        if !section_auth.prefix.matches(dst) {
+        if !section_auth.prefix().matches(dst) {
             error!("Invalid JoinResponse bad prefix: {:?}", section_auth);
             false
-        } else if section_auth.elders.is_empty() {
+        } else if section_auth.elders().is_empty() {
             error!(
                 "Invalid JoinResponse, empty list of Elders: {:?}",
                 section_auth
