@@ -8,24 +8,21 @@
 
 use super::Core;
 use crate::messaging::system::Proposal;
-use crate::routing::{
-    error::Result, peer::PeerUtils, routing_api::command::Command, section::NodeStateUtils,
-    SectionAuthorityProviderUtils,
-};
+use crate::routing::{error::Result, routing_api::command::Command};
 use std::{collections::BTreeSet, iter, net::SocketAddr};
 use xor_name::XorName;
 
 impl Core {
     pub(crate) async fn handle_peer_lost(&self, addr: &SocketAddr) -> Result<Vec<Command>> {
-        let name = if let Some(peer) = self.section.find_joined_member_by_addr(addr) {
+        let name = if let Some(peer) = self.network_knowledge.find_joined_member_by_addr(addr) {
             debug!("Lost known peer {}", peer);
-            *peer.name()
+            peer.name()
         } else {
             trace!("Lost unknown peer {}", addr);
             return Ok(vec![]);
         };
 
-        if self.is_not_elder() {
+        if self.is_not_elder().await {
             // Adults cannot complain about connectivity.
             return Ok(vec![]);
         }
@@ -47,17 +44,20 @@ impl Core {
         // Don't send the `Offline` proposal to the peer being lost as that send would fail,
         // triggering a chain of further `Offline` proposals.
         let elders: Vec<_> = self
-            .section
+            .network_knowledge
             .authority_provider()
+            .await
             .peers()
-            .filter(|peer| !names.contains(peer.name()))
+            .iter()
+            .filter(|peer| !names.contains(&peer.name()))
+            .cloned()
             .collect();
         let mut result: Vec<Command> = Vec::new();
         for name in names.iter() {
-            if let Some(info) = self.section.members().get(name) {
+            if let Some(info) = self.network_knowledge.members().get(name) {
                 let info = info.leave()?;
                 if let Ok(commands) = self
-                    .send_proposal(elders.clone(), Proposal::Offline(info))
+                    .send_proposal(elders.clone(), Proposal::Offline(info.into_msg()))
                     .await
                 {
                     result.extend(commands);
