@@ -7,34 +7,42 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::super::Core;
-use crate::messaging::{
-    system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId, ElderCandidates, Proposal, SystemMsg},
-    SectionAuthorityProvider,
+use crate::messaging::system::{
+    DkgFailureSig, DkgFailureSigSet, DkgSessionId, Proposal, SystemMsg,
 };
 use crate::routing::{
     dkg::DkgFailureSigSetUtils,
     error::{Error, Result},
+    log_markers::LogMarker,
+    network_knowledge::{ElderCandidates, SectionAuthorityProvider, SectionKeyShare},
     routing_api::command::Command,
-    section::SectionKeyShare,
-    SectionAuthorityProviderUtils,
+    Peer,
 };
 use bls_dkg::key_gen::message::Message as DkgMessage;
-use std::collections::BTreeSet;
-use xor_name::XorName;
+use std::{
+    collections::{BTreeMap, BTreeSet},
+    net::SocketAddr,
+};
+use xor_name::{Prefix, XorName};
 
 impl Core {
     pub(crate) async fn handle_dkg_start(
         &self,
         session_id: DkgSessionId,
-        elder_candidates: ElderCandidates,
+        prefix: Prefix,
+        elders: BTreeMap<XorName, SocketAddr>,
     ) -> Result<Vec<Command>> {
+        let elder_candidates = ElderCandidates::new(
+            prefix,
+            elders.into_iter().map(|(name, addr)| Peer::new(name, addr)),
+        );
         trace!("Received DkgStart for {:?}", elder_candidates);
         self.dkg_voter
             .start(
-                &self.node,
+                &self.node.read().await.clone(),
                 session_id,
                 elder_candidates,
-                *self.section_chain().last_key(),
+                self.network_knowledge().section_key().await,
             )
             .await
     }
@@ -45,14 +53,19 @@ impl Core {
         message: DkgMessage,
         sender: XorName,
     ) -> Result<Vec<Command>> {
-        trace!("handle DKG message {:?} from {}", message, sender);
+        trace!(
+            "{} {:?} from {}",
+            LogMarker::DkgMessageHandling,
+            message,
+            sender
+        );
 
         self.dkg_voter
             .process_message(
-                &self.node,
+                &self.node.read().await.clone(),
                 &session_id,
                 message,
-                *self.section_chain().last_key(),
+                self.network_knowledge().section_key().await,
             )
             .await
     }
@@ -77,15 +90,16 @@ impl Core {
         sender: &XorName,
         failure_set: &DkgFailureSigSet,
     ) -> Result<Vec<Command>> {
-        if self.section.members().get(sender).is_none() {
+        if self.network_knowledge.members().get(sender).is_none() {
             return Err(Error::InvalidSrcLocation);
         }
 
-        let generation = self.section.chain().main_branch_len() as u64;
+        let generation = self.network_knowledge.chain_len().await;
 
         let elder_candidates = if let Some(elder_candidates) = self
-            .section
-            .promote_and_demote_elders(&self.node.name(), &BTreeSet::new())
+            .network_knowledge
+            .promote_and_demote_elders(&self.node.read().await.name(), &BTreeSet::new())
+            .await
             .into_iter()
             .find(|elder_candidates| failure_set.verify(elder_candidates, generation))
         {
@@ -127,23 +141,38 @@ impl Core {
         section_auth: SectionAuthorityProvider,
         key_share: SectionKeyShare,
     ) -> Result<Vec<Command>> {
-        let proposal = Proposal::SectionInfo(section_auth);
-        let recipients: Vec<_> = self.section.authority_provider().peers().collect();
-        let result = self.send_proposal_with(recipients, proposal, &key_share);
+        trace!(
+            "{} public_key={:?}",
+            LogMarker::HandlingDkgSuccessfulOutcome,
+            key_share.public_key_set.public_key()
+        );
 
-        let public_key = key_share.public_key_set.public_key();
+        // Add our new keyshare to our cache, we will then use
+        // it to sign any msg that needs section agreement.
+        self.section_keys_provider.insert(key_share.clone()).await;
 
-        self.section_keys_provider.insert_dkg_outcome(key_share);
-
-        if self.section.chain().has_key(&public_key) {
-            self.section_keys_provider.finalise_dkg(&public_key).await
+        let snapshot = self.state_snapshot().await;
+        //
+        if self
+            .network_knowledge
+            .skip_section_info_agreement(key_share.public_key_set.public_key())
+            .await
+        {
+            self.update_self_for_new_node_state_and_fire_events(snapshot)
+                .await
+        } else {
+            let proposal = Proposal::SectionInfo(section_auth.into_msg());
+            let recipients: Vec<_> = self.network_knowledge.authority_provider().await.peers();
+            self.send_proposal_with(recipients, proposal, &key_share)
+                .await
         }
-
-        result
     }
 
-    pub(crate) fn handle_dkg_failure(&self, failure_set: DkgFailureSigSet) -> Result<Command> {
+    pub(crate) async fn handle_dkg_failure(
+        &self,
+        failure_set: DkgFailureSigSet,
+    ) -> Result<Command> {
         let node_msg = SystemMsg::DkgFailureAgreement(failure_set);
-        self.send_message_to_our_elders(node_msg)
+        self.send_message_to_our_elders(node_msg).await
     }
 }
