@@ -10,18 +10,34 @@ use super::super::Core;
 use crate::messaging::{
     system::{
         JoinAsRelocatedRequest, JoinAsRelocatedResponse, JoinRejectionReason, JoinRequest,
-        JoinResponse, Peer, SystemMsg,
+        JoinResponse, SystemMsg,
     },
     WireMsg,
 };
 use crate::routing::{
-    error::Result, log_markers::LogMarker, peer::PeerUtils, relocation::RelocatePayloadUtils,
-    routing_api::command::Command, FIRST_SECTION_MAX_AGE, FIRST_SECTION_MIN_AGE, MIN_ADULT_AGE,
+    error::{Error, Result},
+    log_markers::LogMarker,
+    relocation::RelocatePayloadUtils,
+    routing_api::command::Command,
+    Peer, FIRST_SECTION_MAX_AGE, MIN_ADULT_AGE,
 };
 use bls::PublicKey as BlsPublicKey;
 
 // Message handling
 impl Core {
+    /// Check if we already have this peer in our section
+    fn peer_is_already_a_member(&self, peer: &Peer) -> bool {
+        if self.network_knowledge.members().is_joined(&peer.name()) {
+            debug!(
+                "Ignoring JoinRequest from {} - already member of our section.",
+                peer
+            );
+            return true;
+        }
+
+        false
+    }
+
     pub(crate) async fn handle_join_request(
         &self,
         peer: Peer,
@@ -29,12 +45,19 @@ impl Core {
     ) -> Result<Vec<Command>> {
         debug!("Received {:?} from {}", join_request, peer);
 
-        let section_key_matches = join_request.section_key == *self.section.chain().last_key();
+        let _permit = self
+            .current_joins_semaphore
+            .acquire()
+            .await
+            .map_err(|_| Error::PermitAcquisitionFailed)?;
 
-        // Ignore `JoinRequest` if we are not elder unless the join request
-        // is outdated in which case we reply with `BootstrapResponse::Join`
-        // with the up-to-date info (see `handle_join_request`).
-        if self.is_not_elder() && section_key_matches {
+        let our_section_key = self.network_knowledge.section_key().await;
+        let section_key_matches = join_request.section_key == our_section_key;
+
+        // Ignore `JoinRequest` if we are not elder, unless the join request
+        // is outdated in which case we'll reply with `JoinResponse::Retry`
+        // with the up-to-date info.
+        if self.is_not_elder().await && section_key_matches {
             // Note: We don't bounce this message because the current bounce-resend
             // mechanism wouldn't preserve the original SocketAddr which is needed for
             // properly handling this message.
@@ -43,14 +66,30 @@ impl Core {
             return Ok(vec![]);
         }
 
-        if self.section.members().is_joined(peer.name()) {
-            debug!(
-                "Ignoring JoinRequest from {} - already member of our section.",
-                peer
-            );
+        if self.peer_is_already_a_member(&peer) {
             return Ok(vec![]);
         }
 
+        let our_prefix = self.network_knowledge.prefix().await;
+        if !our_prefix.matches(&peer.name()) {
+            debug!(
+                "Redirecting JoinRequest from {} - name doesn't match our prefix {:?}.",
+                peer, our_prefix
+            );
+
+            let retry_sap = self.matching_section(&peer.name()).await?;
+
+            let node_msg =
+                SystemMsg::JoinResponse(Box::new(JoinResponse::Redirect(retry_sap.into_msg())));
+
+            trace!("Sending {:?} to {}", node_msg, peer);
+            trace!("{}", LogMarker::SendJoinRedirected);
+            return Ok(vec![
+                self.send_direct_message(peer, node_msg, our_section_key)
+                    .await?,
+            ]);
+        }
+
         if !*self.joins_allowed.read().await {
             debug!(
                 "Rejecting JoinRequest from {} - joins currently not allowed.",
@@ -63,90 +102,74 @@ impl Core {
             trace!("{}", LogMarker::SendJoinsDisallowed);
 
             trace!("Sending {:?} to {}", node_msg, peer);
-            return Ok(vec![self.send_direct_message(
-                (*peer.name(), *peer.addr()),
-                node_msg,
-                *self.section.chain().last_key(),
-            )?]);
+            return Ok(vec![
+                self.send_direct_message(peer, node_msg, our_section_key)
+                    .await?,
+            ]);
         }
 
-        if !section_key_matches || !self.section.prefix().matches(peer.name()) {
-            if section_key_matches {
-                debug!(
-                    "JoinRequest from {} - name doesn't match our prefix {:?}.",
-                    peer,
-                    self.section.prefix()
-                );
-            } else {
-                debug!(
-                    "JoinRequest from {} - doesn't have our latest section_key {:?}, presented {:?}.",
-                    peer,
-                    self.section.chain().last_key(),
-                    join_request.section_key
-                );
-            }
-
-            let retry_sap = self.matching_section(peer.name())?;
-
-            let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(retry_sap)));
-            trace!("Sending {:?} to {}", node_msg, peer);
-            trace!("{}", LogMarker::SendJoinRetry);
-            return Ok(vec![self.send_direct_message(
-                (*peer.name(), *peer.addr()),
-                node_msg,
-                *self.section.chain().last_key(),
-            )?]);
-        }
-
-        // Start as Adult as long as passed resource signed.
-        let mut age = MIN_ADULT_AGE;
-
         // During the first section, node shall use ranged age to avoid too many nodes got
         // relocated at the same time. After the first section got split, nodes shall only
         // start with age of MIN_ADULT_AGE
-        if self.section.prefix().is_empty() {
-            if peer.age() < FIRST_SECTION_MIN_AGE || peer.age() > FIRST_SECTION_MAX_AGE {
-                let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(
-                    self.section.authority_provider().clone(),
-                )));
-                trace!("{}", LogMarker::SendJoinRetry);
-                trace!("New node in first section should join with age greater than MIN_ADULT_AGE. Sending {:?} to {}", node_msg, peer);
-                return Ok(vec![self.send_direct_message(
-                    (*peer.name(), *peer.addr()),
-                    node_msg,
-                    *self.section.chain().last_key(),
-                )?]);
+        let expected_age: u8 = if our_prefix.is_empty() {
+            let section_members = self.network_knowledge.active_members().await.len();
+            // Calculate a deterministic value based on peer's address
+            // within the range [FIRST_SECTION_MIN_AGE, FIRST_SECTION_MAX_AGE].
+            // let value: u8 = bincode::serialize(&peer.addr())?.iter().sum();
+            // let range = FIRST_SECTION_MAX_AGE - FIRST_SECTION_MIN_AGE;
+
+            FIRST_SECTION_MAX_AGE - section_members as u8 * 2
+        } else {
+            MIN_ADULT_AGE
+        };
+
+        if !section_key_matches || peer.age() != expected_age {
+            if !section_key_matches {
+                trace!("{}", LogMarker::SendJoinRetryNotCorrectKey);
+                trace!(
+                    "JoinRequest from {} doesn't have our latest section_key {:?}, presented {:?}.",
+                    peer,
+                    our_section_key,
+                    join_request.section_key
+                );
             } else {
-                age = peer.age();
+                trace!("{}", LogMarker::SendJoinRetryAgeIssue);
+                trace!(
+                    "JoinReequest from {} doesn't have the expected age {:?}, expected age {}",
+                    peer,
+                    peer.age(),
+                    expected_age,
+                );
             }
-        } else if peer.age() != MIN_ADULT_AGE {
-            // After section split, new node has to join with age of MIN_ADULT_AGE.
-            let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Retry(
-                self.section.authority_provider().clone(),
-            )));
 
-            trace!("{}", LogMarker::SendJoinRetry);
+            // TODO: once we keep a DAG for chains, return only a delta
+            // from provided section key rather than the chain from genesis key.
+            let proof_chain = self.network_knowledge.chain().await;
+            let signed_sap = self
+                .network_knowledge
+                .section_signed_authority_provider()
+                .await;
 
-            trace!("New node after section split must join with age of MIN_ADULT_AGE. Sending {:?} to {}", node_msg, peer);
-            return Ok(vec![self.send_direct_message(
-                (*peer.name(), *peer.addr()),
-                node_msg,
-                *self.section.chain().last_key(),
-            )?]);
-        }
+            let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Retry {
+                section_auth: signed_sap.value.into_msg(),
+                section_signed: signed_sap.sig,
+                proof_chain,
+                expected_age,
+            }));
 
-        // Requires the node name matches the age.
-        if age != peer.age() {
-            debug!(
-                "Ignoring JoinRequest from {} - required age {:?} not presented.",
-                peer, age,
-            );
-            return Ok(vec![]);
+            trace!("Sending {:?} to {}", node_msg, peer);
+            return Ok(vec![
+                self.send_direct_message(peer, node_msg, our_section_key)
+                    .await?,
+            ]);
         }
 
         // Require resource signed if joining as a new node.
         if let Some(response) = join_request.resource_proof_response {
-            if !self.validate_resource_proof_response(peer.name(), response) {
+            if !self
+                .validate_resource_proof_response(&peer.name(), response)
+                .await
+            {
                 debug!(
                     "Ignoring JoinRequest from {} - invalid resource signed response",
                     peer
@@ -155,22 +178,19 @@ impl Core {
             }
         } else {
             // Do reachability check only for the initial join request
-            let cmd = if self.comm.is_reachable(peer.addr()).await.is_err() {
+            let cmd = if self.comm.is_reachable(&peer.addr()).await.is_err() {
                 let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Rejected(
-                    JoinRejectionReason::NodeNotReachable(*peer.addr()),
+                    JoinRejectionReason::NodeNotReachable(peer.addr()),
                 )));
 
                 trace!("{}", LogMarker::SendJoinRejected);
 
                 trace!("Sending {:?} to {}", node_msg, peer);
-                self.send_direct_message(
-                    (*peer.name(), *peer.addr()),
-                    node_msg,
-                    *self.section.chain().last_key(),
-                )?
+                self.send_direct_message(peer, node_msg, our_section_key)
+                    .await?
             } else {
                 // It's reachable, let's then send the proof challenge
-                self.send_resource_proof_challenge(&peer)?
+                self.send_resource_proof_challenge(peer).await?
             };
 
             return Ok(vec![cmd]);
@@ -189,54 +209,67 @@ impl Core {
         join_request: JoinAsRelocatedRequest,
         known_keys: Vec<BlsPublicKey>,
     ) -> Result<Vec<Command>> {
+        let _permit = self
+            .current_joins_semaphore
+            .acquire()
+            .await
+            .map_err(|_| Error::PermitAcquisitionFailed)?;
+
         debug!("Received {:?} from {}", join_request, peer);
         let relocate_payload = if let Some(relocate_payload) = join_request.relocate_payload {
             relocate_payload
         } else {
             // Do reachability check
-            let node_msg = if self.comm.is_reachable(peer.addr()).await.is_err() {
+            let node_msg = if self.comm.is_reachable(&peer.addr()).await.is_err() {
                 SystemMsg::JoinAsRelocatedResponse(Box::new(
-                    JoinAsRelocatedResponse::NodeNotReachable(*peer.addr()),
+                    JoinAsRelocatedResponse::NodeNotReachable(peer.addr()),
                 ))
             } else {
                 SystemMsg::JoinAsRelocatedResponse(Box::new(JoinAsRelocatedResponse::Retry(
-                    self.section.authority_provider().clone(),
+                    self.network_knowledge.authority_provider().await.into_msg(),
                 )))
             };
             trace!("{}", LogMarker::SendJoinAsRelocatedResponse);
 
             trace!("Sending {:?} to {}", node_msg, peer);
-            return Ok(vec![self.send_direct_message(
-                (*peer.name(), *peer.addr()),
-                node_msg,
-                *self.section.chain().last_key(),
-            )?]);
+            return Ok(vec![
+                self.send_direct_message(
+                    peer,
+                    node_msg,
+                    self.network_knowledge.section_key().await,
+                )
+                .await?,
+            ]);
         };
 
-        if !self.section.prefix().matches(peer.name())
-            || join_request.section_key != *self.section.chain().last_key()
+        if !self.network_knowledge.prefix().await.matches(&peer.name())
+            || join_request.section_key != self.network_knowledge.section_key().await
         {
             debug!(
                 "JoinAsRelocatedRequest from {} - name doesn't match our prefix {:?}.",
                 peer,
-                self.section.prefix()
+                self.network_knowledge.prefix().await
             );
 
-            let node_msg = SystemMsg::JoinAsRelocatedResponse(Box::new(
-                JoinAsRelocatedResponse::Retry(self.section.authority_provider().clone()),
-            ));
+            let node_msg =
+                SystemMsg::JoinAsRelocatedResponse(Box::new(JoinAsRelocatedResponse::Retry(
+                    self.network_knowledge.authority_provider().await.into_msg(),
+                )));
 
             trace!("{} b", LogMarker::SendJoinAsRelocatedResponse);
 
             trace!("Sending {:?} to {}", node_msg, peer);
-            return Ok(vec![self.send_direct_message(
-                (*peer.name(), *peer.addr()),
-                node_msg,
-                *self.section.chain().last_key(),
-            )?]);
+            return Ok(vec![
+                self.send_direct_message(
+                    peer,
+                    node_msg,
+                    self.network_knowledge.section_key().await,
+                )
+                .await?,
+            ]);
         }
 
-        if self.section.members().is_joined(peer.name()) {
+        if self.network_knowledge.members().is_joined(&peer.name()) {
             debug!(
                 "Ignoring JoinAsRelocatedRequest from {} - already member of our section.",
                 peer
@@ -244,7 +277,7 @@ impl Core {
             return Ok(vec![]);
         }
 
-        if !relocate_payload.verify_identity(peer.name()) {
+        if !relocate_payload.verify_identity(&peer.name()) {
             debug!(
                 "Ignoring JoinAsRelocatedRequest from {} - invalid signature.",
                 peer
@@ -254,13 +287,13 @@ impl Core {
 
         let details = relocate_payload.relocate_details()?;
 
-        if !self.section.prefix().matches(&details.dst) {
+        if !self.network_knowledge.prefix().await.matches(&details.dst) {
             debug!(
                 "Ignoring JoinAsRelocatedRequest from {} - destination {} doesn't match \
                          our prefix {:?}.",
                 peer,
                 details.dst,
-                self.section.prefix()
+                self.network_knowledge.prefix().await
             );
             return Ok(vec![]);
         }
@@ -300,7 +333,7 @@ impl Core {
         let dst_key = Some(details.dst_key);
 
         if self
-            .section
+            .network_knowledge
             .members()
             .is_relocated_to_our_section(&details.pub_id)
         {
