@@ -27,9 +27,10 @@ use crate::messaging::{
 use crate::routing::{
     log_markers::LogMarker,
     messages::{NodeMsgAuthorityUtils, WireMsgUtils},
+    network_knowledge::SectionPeers,
     relocation::RelocateState,
     routing_api::command::Command,
-    Error, Event, MessageReceived, Result, SectionAuthorityProviderUtils,
+    Error, Event, MessageReceived, Peer, Result,
 };
 use crate::types::{Chunk, Keypair, PublicKey};
 use bls::PublicKey as BlsPublicKey;
@@ -43,14 +44,14 @@ impl Core {
     #[instrument(skip(self, original_bytes))]
     pub(crate) async fn handle_message(
         &self,
-        sender: SocketAddr,
+        sender_addr: SocketAddr,
         wire_msg: WireMsg,
         original_bytes: Option<Bytes>,
     ) -> Result<Vec<Command>> {
         let mut cmds = vec![];
         trace!("handling msg");
         // Apply backpressure if needed.
-        if let Some(load_report) = self.comm.check_strain(sender).await {
+        if let Some(load_report) = self.comm.check_strain(sender_addr).await {
             let msg_src = wire_msg.msg_kind().src();
             cmds.push(Command::PrepareNodeMsgToSend {
                 msg: SystemMsg::BackPressure(load_report),
@@ -62,7 +63,7 @@ impl Core {
         let payload = wire_msg.payload.clone();
         let msg_id = wire_msg.msg_id();
 
-        let message_type = match wire_msg.clone().into_message() {
+        let message_type = match wire_msg.into_message() {
             Ok(message_type) => message_type,
             Err(error) => {
                 error!(
@@ -82,15 +83,20 @@ impl Core {
             } => {
                 // Let's now verify the section key in the msg authority is trusted
                 // based on our current knowledge of the network and sections chains.
-                let mut known_keys: Vec<BlsPublicKey> =
-                    self.section.chain().keys().copied().collect();
-                known_keys.extend(self.network.section_keys());
-                known_keys.push(*self.section.genesis_key());
+                let mut known_keys: Vec<BlsPublicKey> = self
+                    .network_knowledge
+                    .chain()
+                    .await
+                    .keys()
+                    .copied()
+                    .collect();
+                known_keys.extend(self.network_knowledge.prefix_map().section_keys());
+                known_keys.push(*self.network_knowledge.genesis_key());
 
                 if !msg_authority.verify_src_section_key_is_known(&known_keys) {
                     warn!(
                         "Untrusted message ({:?}) dropped from {:?}: {:?} ",
-                        msg_id, sender, msg
+                        msg_id, sender_addr, msg
                     );
                     return Ok(cmds);
                 }
@@ -98,14 +104,19 @@ impl Core {
                 trace!(
                     "Trusted msg authority in message ({:?}) from {:?}: {:?}",
                     msg_id,
-                    sender,
+                    sender_addr,
                     msg
                 );
 
+                // TODO: msg_authority.peer() exists, but will error if authority is
+                // section/sectionshare. It's not clear if this would actually
+                // represent a valid peer.
+                let sender = Peer::new(msg_authority.name(), sender_addr);
+
                 // Let's check for entropy before we proceed further
                 // Adult nodes don't need to carry out entropy checking,
                 // however the message shall always be handled.
-                if self.is_elder() {
+                if self.is_elder().await {
                     // For the case of receiving a join request not matching our prefix,
                     // we just let the join request handler to deal with it later on.
                     // We also skip AE check on Anti-Entropy messages
@@ -136,7 +147,7 @@ impl Core {
                                         &msg_authority.src_location(),
                                         &dst_section_pk,
                                         dst_location.name(),
-                                        sender,
+                                        &sender,
                                     )
                                     .await?
                                 {
@@ -174,27 +185,28 @@ impl Core {
                     None => {
                         error!(
                             "Service msg has been dropped since {:?} is not a valid msg to send from a client {}.",
-                            msg, sender
+                            msg, sender_addr
                         );
                         return Ok(vec![]);
                     }
                 };
-                let user = match self.comm.get_connection_id(&sender).await {
+                let user = match self.comm.get_peer_connection_id(&sender_addr).await {
                     Some(name) => EndUser(name),
                     None => {
                         error!(
                             "Service msg has been dropped since client connection id for {} was not found: {:?}",
-                            sender, msg
+                            sender_addr, msg
                         );
                         return Ok(cmds);
                     }
                 };
 
+                let sender = Peer::new(user.0, sender_addr);
                 let src_location = SrcLocation::EndUser(user);
 
-                if self.is_not_elder() {
+                if self.is_not_elder().await {
                     trace!("Redirecting from adult to section elders");
-                    cmds.push(self.ae_redirect(sender, &src_location, &wire_msg)?);
+                    cmds.push(self.ae_redirect(sender, &src_location, &wire_msg).await?);
                     return Ok(cmds);
                 }
 
@@ -215,7 +227,7 @@ impl Core {
                         &src_location,
                         &received_section_pk,
                         dst_name,
-                        sender,
+                        &sender,
                     )
                     .await?
                 {
@@ -238,7 +250,7 @@ impl Core {
     #[allow(clippy::too_many_arguments)]
     pub(crate) async fn handle_system_message(
         &self,
-        sender: SocketAddr,
+        sender: Peer,
         msg_id: MessageId,
         mut msg_authority: NodeMsgAuthority,
         dst_location: DstLocation,
@@ -253,40 +265,10 @@ impl Core {
             .aggregate_message_and_stop(&mut msg_authority, payload)
             .await
         {
-            Ok(false) => match msg {
-                SystemMsg::NodeCmd(_)
-                | SystemMsg::JoinResponse(_)
-                | SystemMsg::JoinRequest(_)
-                | SystemMsg::JoinAsRelocatedRequest(_)
-                | SystemMsg::DkgStart { .. }
-                | SystemMsg::DkgFailureAgreement(_)
-                | SystemMsg::DkgMessage { .. }
-                | SystemMsg::DkgFailureObservation { .. }
-                | SystemMsg::Propose { .. }
-                | SystemMsg::NodeQuery(_)
-                | SystemMsg::NodeQueryResponse { .. } => {
-                    let cmd = Command::HandleNonBlockingMessage {
-                        msg_id,
-                        msg,
-                        msg_authority,
-                        dst_location,
-                        sender,
-                        known_keys,
-                    };
-
-                    Ok(vec![cmd])
-                }
-                _ => {
-                    let cmd = Command::HandleBlockingMessage {
-                        sender,
-                        msg_id,
-                        msg,
-                        msg_authority,
-                    };
-
-                    Ok(vec![cmd])
-                }
-            },
+            Ok(false) => {
+                self.handle_valid_msg(msg_id, msg_authority, dst_location, msg, sender, known_keys)
+                    .await
+            }
             Err(Error::InvalidSignatureShare) => {
                 warn!(
                     "Invalid signature on received system message, dropping the message: {:?}",
@@ -298,50 +280,20 @@ impl Core {
         }
     }
 
-    // Handler for node messages which have successfully
+    // Handler for data messages which have successfully
     // passed all signature checks and msg verifications
-    pub(crate) async fn handle_blocking_message(
-        &mut self,
-        sender: SocketAddr,
+    pub(crate) async fn handle_valid_msg(
+        &self,
         msg_id: MessageId,
         msg_authority: NodeMsgAuthority,
+        dst_location: DstLocation,
         node_msg: SystemMsg,
+        sender: Peer,
+        known_keys: Vec<BlsPublicKey>,
     ) -> Result<Vec<Command>> {
         let src_name = msg_authority.name();
-
-        trace!("Handling blocking system message");
+        trace!("Handling non blocking message");
         match node_msg {
-            SystemMsg::AntiEntropyRetry {
-                section_auth,
-                section_signed,
-                proof_chain,
-                bounced_msg,
-            } => {
-                trace!("Handling msg: AE-Retry from {}: {:?}", sender, msg_id,);
-                self.handle_anti_entropy_retry_msg(
-                    section_auth,
-                    section_signed,
-                    proof_chain,
-                    bounced_msg,
-                    sender,
-                    src_name,
-                )
-                .await
-            }
-            SystemMsg::AntiEntropyRedirect {
-                section_auth,
-                section_signed,
-                bounced_msg,
-            } => {
-                trace!("Handling msg: AE-Redirect from {}: {:?}", sender, msg_id);
-                self.handle_anti_entropy_redirect_msg(
-                    section_auth,
-                    section_signed,
-                    bounced_msg,
-                    sender,
-                )
-                .await
-            }
             SystemMsg::AntiEntropyUpdate {
                 section_auth,
                 section_signed,
@@ -350,24 +302,17 @@ impl Core {
             } => {
                 trace!("Handling msg: AE-Update from {}: {:?}", sender, msg_id,);
                 self.handle_anti_entropy_update_msg(
-                    section_auth,
+                    section_auth.into_state(),
                     section_signed,
                     proof_chain,
-                    members,
-                    sender,
+                    members.map(|members| {
+                        SectionPeers::new(
+                            members.into_iter().map(|member| member.into_authed_state()),
+                        )
+                    }),
                 )
                 .await
             }
-            SystemMsg::AntiEntropyProbe(_dst) => {
-                trace!("Received Probe message from {}: {:?}", sender, msg_id);
-                Ok(vec![])
-            }
-            SystemMsg::BackPressure(load_report) => {
-                trace!("Handling msg: BackPressure from {}: {:?}", sender, msg_id);
-                // #TODO: Factor in med/long term backpressure into general node liveness calculations
-                self.comm.regulate(sender, load_report).await;
-                Ok(vec![])
-            }
             SystemMsg::Relocate(ref details) => {
                 trace!("Handling msg: Relocate from {}: {:?}", sender, msg_id);
                 if let NodeMsgAuthority::Section(section_signed) = msg_authority {
@@ -394,7 +339,7 @@ impl Core {
                     sender,
                     msg_id
                 );
-                if self.is_not_elder() {
+                if self.is_not_elder().await {
                     return Ok(vec![]);
                 }
 
@@ -403,10 +348,10 @@ impl Core {
             SystemMsg::JoinAsRelocatedResponse(join_response) => {
                 trace!("Handling msg: JoinAsRelocatedResponse from {}", sender);
                 if let Some(RelocateState::InProgress(ref mut joining_as_relocated)) =
-                    self.relocate_state.as_mut()
+                    *self.relocate_state.write().await
                 {
                     if let Some(cmd) = joining_as_relocated
-                        .handle_join_response(*join_response, sender)
+                        .handle_join_response(*join_response, sender.addr())
                         .await?
                     {
                         return Ok(vec![cmd]);
@@ -428,30 +373,46 @@ impl Core {
                 );
                 Ok(vec![])
             }
-            _ => {
-                warn!(
-                    "!!! Unexpected SystemMsg handled at verified non thread safe nodemsg handling: {:?}",
-                    node_msg
-                );
+            SystemMsg::AntiEntropyRetry {
+                section_auth,
+                section_signed,
+                proof_chain,
+                bounced_msg,
+            } => {
+                trace!("Handling msg: AE-Retry from {}: {:?}", sender, msg_id,);
+                self.handle_anti_entropy_retry_msg(
+                    section_auth.into_state(),
+                    section_signed,
+                    proof_chain,
+                    bounced_msg,
+                    sender,
+                )
+                .await
+            }
+            SystemMsg::AntiEntropyRedirect {
+                section_auth,
+                section_signed,
+                bounced_msg,
+            } => {
+                trace!("Handling msg: AE-Redirect from {}: {:?}", sender, msg_id);
+                self.handle_anti_entropy_redirect_msg(
+                    section_auth.into_state(),
+                    section_signed,
+                    bounced_msg,
+                    sender,
+                )
+                .await
+            }
+            SystemMsg::AntiEntropyProbe(_dst) => {
+                trace!("Received Probe message from {}: {:?}", sender, msg_id);
+                Ok(vec![])
+            }
+            SystemMsg::BackPressure(load_report) => {
+                trace!("Handling msg: BackPressure from {}: {:?}", sender, msg_id);
+                // #TODO: Factor in med/long term backpressure into general node liveness calculations
+                self.comm.regulate(sender.addr(), load_report).await;
                 Ok(vec![])
             }
-        }
-    }
-
-    // Handler for data messages which have successfully
-    // passed all signature checks and msg verifications
-    pub(crate) async fn handle_non_blocking_message(
-        &self,
-        msg_id: MessageId,
-        msg_authority: NodeMsgAuthority,
-        dst_location: DstLocation,
-        node_msg: SystemMsg,
-        sender: SocketAddr,
-        known_keys: Vec<BlsPublicKey>,
-    ) -> Result<Vec<Command>> {
-        let src_name = msg_authority.name();
-        trace!("Handling non blocking message");
-        match node_msg {
             SystemMsg::JoinResponse(join_response) => {
                 debug!(
                     "Ignoring unexpected join response message: {:?}",
@@ -465,47 +426,43 @@ impl Core {
             }
             SystemMsg::JoinRequest(join_request) => {
                 trace!("Handling msg: JoinRequest from {}", sender);
-                self.handle_join_request(msg_authority.peer(sender)?, *join_request)
-                    .await
+                self.handle_join_request(sender, *join_request).await
             }
             SystemMsg::JoinAsRelocatedRequest(join_request) => {
                 trace!("Handling msg: JoinAsRelocatedRequest from {}", sender);
-                if self.is_not_elder()
-                    && join_request.section_key == *self.section.chain().last_key()
+                if self.is_not_elder().await
+                    && join_request.section_key == self.network_knowledge.section_key().await
                 {
                     return Ok(vec![]);
                 }
 
-                self.handle_join_as_relocated_request(
-                    msg_authority.peer(sender)?,
-                    *join_request,
-                    known_keys,
-                )
-                .await
+                self.handle_join_as_relocated_request(sender, *join_request, known_keys)
+                    .await
             }
             SystemMsg::Propose {
                 proposal,
                 sig_share,
             } => {
-                if self.is_not_elder() {
+                if self.is_not_elder().await {
                     trace!("Dropping Propose msg from {}: {:?}", sender, msg_id);
                     return Ok(vec![]);
                 }
 
                 trace!("Handling msg: Propose from {}: {:?}", sender, msg_id);
-                self.handle_proposal(msg_id, proposal, sig_share, src_name, sender)
+                self.handle_proposal(msg_id, proposal, sig_share, sender)
                     .await
             }
             SystemMsg::DkgStart {
                 session_id,
-                elder_candidates,
+                prefix,
+                elders,
             } => {
                 trace!("Handling msg: Dkg-Start from {}", sender);
-                if !elder_candidates.elders.contains_key(&self.node.name()) {
+                if !elders.contains_key(&self.node.read().await.name()) {
                     return Ok(vec![]);
                 }
 
-                self.handle_dkg_start(session_id, elder_candidates).await
+                self.handle_dkg_start(session_id, prefix, elders).await
             }
             SystemMsg::DkgMessage {
                 session_id,
@@ -517,7 +474,8 @@ impl Core {
                     message,
                     sender
                 );
-                self.handle_dkg_message(session_id, message, src_name).await
+                self.handle_dkg_message(session_id, message, sender.name())
+                    .await
             }
             SystemMsg::DkgFailureObservation {
                 session_id,
@@ -544,7 +502,7 @@ impl Core {
                             msg_id
                         );
 
-                        return if self.is_elder() {
+                        return if self.is_elder().await {
                             self.republish_chunk(chunk).await
                         } else {
                             // We are an adult here, so just store away!
@@ -619,28 +577,6 @@ impl Core {
                 )
                 .await
             }
-            SystemMsg::NodeMsgError {
-                error,
-                correlation_id,
-            } => {
-                trace!(
-                    "From {:?}({:?}), received error {:?} correlated to {:?}, targeting {:?}",
-                    msg_authority.src_location(),
-                    msg_id,
-                    error,
-                    correlation_id,
-                    dst_location
-                );
-                Ok(vec![])
-            }
-            _ => {
-                warn!(
-                    "Non data message provided to data message handler {:?}",
-                    node_msg
-                );
-                // do nothing
-                Ok(vec![])
-            }
         }
     }
 
@@ -648,7 +584,7 @@ impl Core {
         let mut cmds = vec![];
         if let Some(level) = level {
             info!("Storage has now passed {} % used.", 10 * level.value());
-            let node_id = PublicKey::from(self.node().keypair.public);
+            let node_id = PublicKey::from(self.node.read().await.keypair.public);
             let node_xorname = XorName::from(node_id);
 
             // we ask the section to record the new level reached
@@ -660,7 +596,7 @@ impl Core {
 
             let dst = DstLocation::Section {
                 name: node_xorname,
-                section_pk: self.section.section_auth.value.section_key(),
+                section_pk: self.network_knowledge.section_key().await,
             };
 
             cmds.push(Command::PrepareNodeMsgToSend { msg, dst });
@@ -670,7 +606,7 @@ impl Core {
 
     // Locate ideal chunk holders for this chunk, line up wiremsgs for those to instruct them to store the chunk
     async fn republish_chunk(&self, chunk: Chunk) -> Result<Vec<Command>> {
-        if self.is_elder() {
+        if self.is_elder().await {
             let target_holders = self.get_chunk_holder_adults(chunk.name()).await;
             info!(
                 "Republishing chunk {:?} to holders {:?}",
@@ -698,12 +634,12 @@ impl Core {
     ) -> Result<Vec<Command>> {
         let msg_id = MessageId::new();
 
-        let our_name = self.node().name();
+        let our_name = self.node.read().await.name();
 
         // we create a dummy/random dst location,
         // we will set it correctly for each msg and target
         // let name = network.our_name().await;
-        let section_pk = *self.section_chain().last_key();
+        let section_pk = self.network_knowledge().section_key().await;
 
         let dummy_dst_location = DstLocation::Node {
             name: our_name,
@@ -722,7 +658,12 @@ impl Core {
                 section_pk,
             )
         } else {
-            WireMsg::single_src(self.node(), dummy_dst_location, msg, section_pk)
+            WireMsg::single_src(
+                &self.node.read().await.clone(),
+                dummy_dst_location,
+                msg,
+                section_pk,
+            )
         }?;
 
         wire_msg.set_msg_id(msg_id);
@@ -732,7 +673,7 @@ impl Core {
         for target in targets {
             debug!("sending {:?} to {:?}", wire_msg, target);
             let mut wire_msg = wire_msg.clone();
-            let dst_section_pk = self.section_key_by_name(&target);
+            let dst_section_pk = self.section_key_by_name(&target).await;
             wire_msg.set_dst_section_pk(dst_section_pk);
             wire_msg.set_dst_xorname(target);
 
