@@ -8,17 +8,15 @@
 
 use super::Core;
 use crate::messaging::{
-    system::{Peer, Proposal, RelocateDetails, RelocatePromise, SystemMsg},
+    system::{Proposal, RelocateDetails, RelocatePromise, SystemMsg},
     AuthorityProof, SectionAuth,
 };
 use crate::routing::{
     core::bootstrap::JoiningAsRelocated,
     error::Result,
-    peer::PeerUtils,
     relocation::{self, RelocateAction, RelocateDetailsUtils, RelocateState},
     routing_api::command::Command,
-    section::NodeStateUtils,
-    Event, SectionAuthorityProviderUtils, ELDER_SIZE,
+    Event, Peer, ELDER_SIZE,
 };
 use xor_name::XorName;
 
@@ -32,27 +30,32 @@ impl Core {
         let mut commands = vec![];
 
         // Do not carry out relocation when there is not enough elder nodes.
-        if self.section.authority_provider().elder_count() < ELDER_SIZE {
+        if self
+            .network_knowledge
+            .authority_provider()
+            .await
+            .elder_count()
+            < ELDER_SIZE
+        {
             return Ok(commands);
         }
 
         // Consider: Set <= 4, as to not carry out relocations in first 16 sections.
         // TEMP: Do not carry out relocations in the first section
-        if self.section.prefix().bit_count() < 1 {
+        if self.network_knowledge.prefix().await.bit_count() < 1 {
             return Ok(commands);
         }
 
-        let relocations =
-            relocation::actions(&self.section, &self.network, churn_name, churn_signature);
-
-        for (info, action) in relocations {
-            let peer = info.peer;
+        let relocations = relocation::actions(&self.network_knowledge, churn_name, churn_signature);
 
+        for (info, action) in relocations.await {
             // The newly joined node is not being relocated immediately.
-            if peer.name() == churn_name {
+            if &info.name() == churn_name {
                 continue;
             }
 
+            let peer = info.to_peer();
+
             debug!(
                 "Relocating {:?} to {} (on churn of {})",
                 peer,
@@ -61,7 +64,7 @@ impl Core {
             );
 
             commands.extend(
-                self.propose(Proposal::Offline(info.relocate(*action.dst())))
+                self.propose(Proposal::Offline(info.relocate(*action.dst()).into_msg()))
                     .await?,
             );
 
@@ -80,11 +83,11 @@ impl Core {
 
     pub(crate) async fn relocate_rejoining_peer(
         &self,
-        peer: &Peer,
+        peer: Peer,
         age: u8,
     ) -> Result<Vec<Command>> {
         let details =
-            RelocateDetails::with_age(&self.section, &self.network, peer, *peer.name(), age);
+            RelocateDetails::with_age(&self.network_knowledge, &peer, peer.name(), age).await;
 
         trace!(
             "Relocating {:?} to {} with age {} due to rejoin",
@@ -93,16 +96,16 @@ impl Core {
             details.age
         );
 
-        self.send_relocate(*peer, details).await
+        self.send_relocate(peer, details).await
     }
 
     pub(crate) async fn handle_relocate(
-        &mut self,
+        &self,
         relocate_details: RelocateDetails,
         node_msg: SystemMsg,
         section_auth: AuthorityProof<SectionAuth>,
     ) -> Result<Option<Command>> {
-        if relocate_details.pub_id != self.node.name() {
+        if relocate_details.pub_id != self.node.read().await.name() {
             // This `Relocate` message is not for us - it's most likely a duplicate of a previous
             // message that we already handled.
             return Ok(None);
@@ -113,7 +116,7 @@ impl Core {
             relocate_details.dst
         );
 
-        match self.relocate_state {
+        match *self.relocate_state.read().await {
             Some(RelocateState::InProgress(_)) => {
                 trace!("Ignore Relocate - relocation already in progress");
                 return Ok(None);
@@ -121,7 +124,7 @@ impl Core {
             Some(RelocateState::Delayed(_)) => (),
             None => {
                 self.send_event(Event::RelocationStarted {
-                    previous_name: self.node.name(),
+                    previous_name: self.node.read().await.name(),
                 })
                 .await;
             }
@@ -129,10 +132,14 @@ impl Core {
 
         // Create a new instance of JoiningAsRelocated to start the relocation
         // flow. This same instance will handle responses till relocation is complete.
-        let genesis_key = *self.section.genesis_key();
-        let bootstrap_addrs = self.section.authority_provider().addresses();
+        let genesis_key = *self.network_knowledge.genesis_key();
+        let bootstrap_addrs = self
+            .network_knowledge
+            .authority_provider()
+            .await
+            .addresses();
         let mut joining_as_relocated = JoiningAsRelocated::new(
-            self.node.clone(),
+            self.node.read().await.clone(),
             genesis_key,
             relocate_details,
             node_msg,
@@ -141,27 +148,28 @@ impl Core {
 
         let cmd = joining_as_relocated.start(bootstrap_addrs)?;
 
-        self.relocate_state = Some(RelocateState::InProgress(Box::new(joining_as_relocated)));
+        *self.relocate_state.write().await =
+            Some(RelocateState::InProgress(Box::new(joining_as_relocated)));
 
         Ok(Some(cmd))
     }
 
     pub(crate) async fn handle_relocate_promise(
-        &mut self,
+        &self,
         promise: RelocatePromise,
         msg: SystemMsg,
     ) -> Result<Vec<Command>> {
         // Check if we need to filter out the `RelocatePromise`.
-        if promise.name == self.node.name() {
+        if promise.name == self.node.read().await.name() {
             // Promise to relocate us.
-            if self.relocate_state.is_some() {
+            if self.relocate_state.read().await.is_some() {
                 // Already received a promise or already relocating. discard.
                 return Ok(vec![]);
             }
         } else {
             // Promise returned from a node to be relocated, to be exchanged for the actual
             // `Relocate` message.
-            if self.is_not_elder() || self.section.is_elder(&promise.name) {
+            if self.is_not_elder().await || self.network_knowledge.is_elder(&promise.name).await {
                 // If we are not elder, maybe we just haven't processed our promotion yet.
                 // If otherwise they are still elder, maybe we just haven't processed their demotion yet.
                 return Ok(vec![]);
@@ -170,15 +178,15 @@ impl Core {
 
         let mut commands = vec![];
 
-        if promise.name == self.node.name() {
+        if promise.name == self.node.read().await.name() {
             // Store the `RelocatePromise` message and send it back after we are demoted.
             // Keep it around even if we are not elder anymore, in case we need to resend it.
-            match self.relocate_state {
+            match *self.relocate_state.read().await {
                 None => {
                     trace!("Received RelocatePromise to section at {}", promise.dst);
-                    self.relocate_state = Some(RelocateState::Delayed(msg.clone()));
+                    *self.relocate_state.write().await = Some(RelocateState::Delayed(msg.clone()));
                     self.send_event(Event::RelocationStarted {
-                        previous_name: self.node.name(),
+                        previous_name: self.node.read().await.name(),
                     })
                     .await;
                 }
@@ -191,14 +199,14 @@ impl Core {
             }
 
             // We are no longer elder. Send the promise back already.
-            if self.is_not_elder() {
-                commands.push(self.send_message_to_our_elders(msg)?);
+            if self.is_not_elder().await {
+                commands.push(self.send_message_to_our_elders(msg).await?);
             }
 
             return Ok(commands);
         }
 
-        if self.section.is_elder(&promise.name) {
+        if self.network_knowledge.is_elder(&promise.name).await {
             error!(
                 "ignore returned RelocatePromise from {} - node is still elder",
                 promise.name
@@ -206,10 +214,10 @@ impl Core {
             return Ok(commands);
         }
 
-        if let Some(info) = self.section.members().get(&promise.name) {
-            let details =
-                RelocateDetails::new(&self.section, &self.network, &info.peer, promise.dst);
-            commands.extend(self.send_relocate(info.peer, details).await?);
+        if let Some(info) = self.network_knowledge.members().get(&promise.name) {
+            let peer = info.to_peer();
+            let details = RelocateDetails::new(&self.network_knowledge, &peer, promise.dst).await;
+            commands.extend(self.send_relocate(peer, details).await?);
         } else {
             error!(
                 "ignore returned RelocatePromise from {} - unknown node",
