@@ -9,40 +9,37 @@
 //! Relocation related types and utilities.
 
 use crate::messaging::{
-    system::{
-        NodeState, Peer, RelocateDetails, RelocatePayload, RelocatePromise, Section, SystemMsg,
-    },
+    system::{RelocateDetails, RelocatePayload, RelocatePromise, SystemMsg},
     AuthorityProof, SectionAuth,
 };
-use crate::prefix_map::NetworkPrefixMap;
 use crate::routing::{
     core::JoiningAsRelocated,
     ed25519::{self, Keypair, Verifier},
     error::Error,
-    peer::PeerUtils,
-    section::section_authority_provider::SectionAuthorityProviderUtils,
+    network_knowledge::{NetworkKnowledge, NodeState},
+    Peer,
 };
+use async_trait::async_trait;
 use xor_name::XorName;
 
 /// Find all nodes to relocate after a churn event and create the relocate actions for them.
-pub(crate) fn actions(
-    section: &Section,
-    network: &NetworkPrefixMap,
+pub(crate) async fn actions(
+    network_knowledge: &NetworkKnowledge,
     churn_name: &XorName,
     churn_signature: &bls::Signature,
 ) -> Vec<(NodeState, RelocateAction)> {
     // Find the peers that pass the relocation check and take only the oldest ones to avoid
     // relocating too many nodes at the same time.
 
-    let filtered = section
+    let filtered = network_knowledge
         .members()
         .joined()
         .into_iter()
-        .filter(|info| check(info.peer.age(), churn_signature));
+        .filter(|info| check(info.age(), churn_signature));
 
     let candidates: Vec<_> = filtered.collect();
 
-    let max_age = if let Some(age) = candidates.iter().map(|info| (*info).peer.age()).max() {
+    let max_age = if let Some(age) = candidates.iter().map(|info| info.age()).max() {
         age
     } else {
         return vec![];
@@ -51,10 +48,11 @@ pub(crate) fn actions(
     let mut relocating_nodes = vec![];
 
     for node_state in candidates {
-        if node_state.peer.age() == max_age {
+        if node_state.age() == max_age {
+            let peer = node_state.to_peer();
             relocating_nodes.push((
                 node_state,
-                RelocateAction::new(section, network, &node_state.peer, churn_name),
+                RelocateAction::new(network_knowledge, &peer, churn_name).await,
             ))
         }
     }
@@ -64,37 +62,38 @@ pub(crate) fn actions(
 
 /// Details of a relocation: which node to relocate, where to relocate it to and what age it should
 /// get once relocated.
+#[async_trait]
 pub(super) trait RelocateDetailsUtils {
-    fn new(section: &Section, network: &NetworkPrefixMap, peer: &Peer, dst: XorName) -> Self;
+    async fn new(network_knowledge: &NetworkKnowledge, peer: &Peer, dst: XorName) -> Self;
 
-    fn with_age(
-        section: &Section,
-        network: &NetworkPrefixMap,
+    async fn with_age(
+        network_knowledge: &NetworkKnowledge,
         peer: &Peer,
         dst: XorName,
         age: u8,
     ) -> RelocateDetails;
 }
 
+#[async_trait]
 impl RelocateDetailsUtils for RelocateDetails {
-    fn new(section: &Section, network: &NetworkPrefixMap, peer: &Peer, dst: XorName) -> Self {
-        Self::with_age(section, network, peer, dst, peer.age().saturating_add(1))
+    async fn new(network_knowledge: &NetworkKnowledge, peer: &Peer, dst: XorName) -> Self {
+        Self::with_age(network_knowledge, peer, dst, peer.age().saturating_add(1)).await
     }
 
-    fn with_age(
-        section: &Section,
-        network: &NetworkPrefixMap,
+    async fn with_age(
+        network_knowledge: &NetworkKnowledge,
         peer: &Peer,
         dst: XorName,
         age: u8,
     ) -> RelocateDetails {
-        let dst_key = network.section_by_name(&dst).map_or_else(
-            |_| *section.chain().root_key(),
-            |section_auth| section_auth.section_key(),
-        );
+        let root_key = *network_knowledge.genesis_key();
+
+        let dst_key = network_knowledge
+            .section_by_name(&dst)
+            .map_or_else(|_| root_key, |section_auth| section_auth.section_key());
 
         RelocateDetails {
-            pub_id: *peer.name(),
+            pub_id: peer.name(),
             dst,
             dst_key,
             age,
@@ -179,21 +178,20 @@ pub(crate) enum RelocateAction {
 }
 
 impl RelocateAction {
-    pub(crate) fn new(
-        section: &Section,
-        network: &NetworkPrefixMap,
+    pub(crate) async fn new(
+        network_knowledge: &NetworkKnowledge,
         peer: &Peer,
         churn_name: &XorName,
     ) -> Self {
-        let dst = dst(peer.name(), churn_name);
+        let dst = dst(&peer.name(), churn_name);
 
-        if section.is_elder(peer.name()) {
+        if network_knowledge.is_elder(&peer.name()).await {
             RelocateAction::Delayed(RelocatePromise {
-                name: *peer.name(),
+                name: peer.name(),
                 dst,
             })
         } else {
-            RelocateAction::Instant(RelocateDetails::new(section, network, peer, dst))
+            RelocateAction::Instant(RelocateDetails::new(network_knowledge, peer, dst).await)
         }
     }
 
@@ -246,11 +244,11 @@ fn trailing_zeros(bytes: &[u8]) -> u32 {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::messaging::SectionAuthorityProvider;
     use crate::routing::{
-        dkg::test_utils::section_signed, peer::test_utils::arbitrary_unique_peers,
-        routing_api::tests::SecretKeySet, section::NodeStateUtils, SectionAuthorityProviderUtils,
-        ELDER_SIZE, MIN_AGE,
+        dkg::test_utils::section_signed,
+        network_knowledge::peer::test_utils::arbitrary_unique_peers,
+        network_knowledge::SectionAuthorityProvider, routing_api::tests::SecretKeySet, ELDER_SIZE,
+        MIN_AGE,
     };
     use assert_matches::assert_matches;
     use eyre::Result;
@@ -304,29 +302,34 @@ mod tests {
                 .sorted_by_key(|peer| peer.age())
                 .rev()
                 .take(ELDER_SIZE)
-                .copied(),
+                .cloned(),
             Prefix::default(),
             sk_set.public_keys(),
         );
         let section_auth = section_signed(sk, section_auth)?;
 
-        let mut section =
-            Section::new(genesis_pk, SecuredLinkedList::new(genesis_pk), section_auth)?;
+        let network_knowledge = NetworkKnowledge::new(
+            genesis_pk,
+            SecuredLinkedList::new(genesis_pk),
+            section_auth,
+            None,
+        )?;
 
         for peer in &peers {
-            let info = NodeState::joined(*peer, None);
+            let info = NodeState::joined(peer, None);
             let info = section_signed(sk, info)?;
 
-            assert!(section.update_member(info));
+            let res = futures::executor::block_on(network_knowledge.update_member(info));
+            assert!(res);
         }
 
-        let network = NetworkPrefixMap::new(genesis_pk);
-
         // Simulate a churn event whose signature has the given number of trailing zeros.
         let churn_name = rng.gen();
         let churn_signature = signature_with_trailing_zeros(signature_trailing_zeros as u32);
 
-        let actions = actions(&section, &network, &churn_name, &churn_signature);
+        let actions = actions(&network_knowledge, &churn_name, &churn_signature);
+        let actions = futures::executor::block_on(actions);
+
         let actions: Vec<_> = actions
             .into_iter()
             .map(|(_, action)| action)
@@ -343,7 +346,7 @@ mod tests {
         let expected_relocated_peers: Vec<_> = peers
             .iter()
             .filter(|peer| Some(peer.age()) == expected_relocated_age)
-            .sorted_by_key(|peer| *peer.name())
+            .sorted_by_key(|peer| peer.name())
             .collect();
 
         assert_eq!(expected_relocated_peers.len(), actions.len());
@@ -351,9 +354,11 @@ mod tests {
         // Verify the relocate action is correct depending on whether the peer is elder or not.
         // NOTE: `zip` works here, because both collections are sorted by name.
         for (peer, action) in expected_relocated_peers.into_iter().zip(actions) {
-            assert_eq!(peer.name(), action.name());
+            assert_eq!(&peer.name(), action.name());
 
-            if section.is_elder(peer.name()) {
+            let is_elder = futures::executor::block_on(network_knowledge.is_elder(&peer.name()));
+
+            if is_elder {
                 assert_matches!(action, RelocateAction::Delayed(_));
             } else {
                 assert_matches!(action, RelocateAction::Instant(_));
