@@ -9,7 +9,7 @@
 use super::{Command, Event};
 use crate::messaging::{
     data::{ChunkDataExchange, StorageLevel},
-    system::{Section, SystemMsg},
+    system::{Proposal, SystemMsg},
     DstLocation, EndUser, MsgKind, WireMsg,
 };
 use crate::routing::{
@@ -18,26 +18,23 @@ use crate::routing::{
     error::Result,
     log_markers::LogMarker,
     messages::WireMsgUtils,
+    network_knowledge::NetworkKnowledge,
     node::Node,
-    peer::PeerUtils,
-    Error, Prefix, XorName,
+    Error, Peer, Prefix, XorName,
 };
 use crate::types::PublicKey;
 use itertools::Itertools;
 use std::collections::BTreeSet;
-use std::{net::SocketAddr, sync::Arc, time::Duration};
+use std::{sync::Arc, time::Duration};
 use tokio::time::MissedTickBehavior;
-use tokio::{
-    sync::{watch, RwLock},
-    time,
-};
+use tokio::{sync::watch, time};
 use tracing::Instrument;
 
 const PROBE_INTERVAL: Duration = Duration::from_secs(30);
 
 // `Command` Dispatcher.
 pub(super) struct Dispatcher {
-    pub(super) core: RwLock<Core>,
+    pub(super) core: Core,
 
     cancel_timer_tx: watch::Sender<bool>,
     cancel_timer_rx: watch::Receiver<bool>,
@@ -46,7 +43,7 @@ pub(super) struct Dispatcher {
 impl Drop for Dispatcher {
     fn drop(&mut self) {
         // Cancel all scheduled timers including any future ones.
-        let _ = self.cancel_timer_tx.send(true);
+        let _res = self.cancel_timer_tx.send(true);
     }
 }
 
@@ -54,35 +51,31 @@ impl Dispatcher {
     pub(super) fn new(core: Core) -> Self {
         let (cancel_timer_tx, cancel_timer_rx) = watch::channel(false);
         Self {
-            core: RwLock::new(core),
+            core,
             cancel_timer_tx,
             cancel_timer_rx,
         }
     }
 
     pub(super) async fn get_register_storage(&self) -> RegisterStorage {
-        self.core.read().await.register_storage.clone()
+        self.core.register_storage.clone()
     }
 
     pub(super) async fn get_chunk_storage(&self) -> ChunkStore {
-        self.core.read().await.chunk_storage.clone()
+        self.core.chunk_storage.clone()
     }
 
     pub(super) async fn get_chunk_data_of(&self, prefix: &Prefix) -> ChunkDataExchange {
-        self.core.read().await.get_data_of(prefix).await
+        self.core.get_data_of(prefix).await
     }
 
     /// Returns whether the level changed or not.
     pub(super) async fn set_storage_level(&self, node_id: &PublicKey, level: StorageLevel) -> bool {
-        self.core
-            .read()
-            .await
-            .set_storage_level(node_id, level)
-            .await
+        self.core.set_storage_level(node_id, level).await
     }
 
     pub(super) async fn retain_members_only(&self, members: BTreeSet<XorName>) -> Result<()> {
-        self.core.read().await.retain_members_only(members).await
+        self.core.retain_members_only(members).await
     }
 
     /// Handles the given command and transitively any new commands that are
@@ -123,20 +116,19 @@ impl Dispatcher {
 
     pub(super) async fn start_network_probing(self: Arc<Self>) {
         info!("Starting to probe network");
-        let _ = tokio::spawn(async move {
+        let _handle = tokio::spawn(async move {
             let dispatcher = self.clone();
             let mut interval = tokio::time::interval(PROBE_INTERVAL);
             interval.set_missed_tick_behavior(MissedTickBehavior::Skip);
 
             loop {
-                let _ = interval.tick().await;
+                let _instant = interval.tick().await;
 
                 // Send a probe message if we are an elder
-                let core = dispatcher.core.read().await;
-                if core.is_elder() && !core.section().prefix().is_empty() {
-                    match core.generate_probe_message() {
+                let core = &dispatcher.core;
+                if core.is_elder().await && !core.network_knowledge().prefix().await.is_empty() {
+                    match core.generate_probe_message().await {
                         Ok(command) => {
-                            drop(core);
                             info!("Sending ProbeMessage");
                             if let Err(e) = dispatcher.clone().handle_commands(command, None).await
                             {
@@ -152,7 +144,7 @@ impl Dispatcher {
 
     pub(super) async fn write_prefixmap_to_disk(self: Arc<Self>) {
         info!("Writing our PrefixMap to disk");
-        self.clone().core.read().await.write_prefix_map().await
+        self.clone().core.write_prefix_map().await
     }
 
     /// Handles a single command.
@@ -165,20 +157,26 @@ impl Dispatcher {
         // analyzing logs produced by running multiple nodes within the same process, for example
         // from integration tests.
         let span = {
-            let core = self.core.read().await;
+            let core = &self.core;
+
+            let prefix = core.network_knowledge().prefix().await;
+            let is_elder = core.is_elder().await;
+            let section_key = core.network_knowledge().section_key().await;
+            let age = core.node.read().await.age();
             trace_span!(
                 "handle_command",
-                name = %core.node().name(),
-                prefix = format_args!("({:b})", core.section().prefix()),
-                age = core.node().age(),
-                elder = core.is_elder(),
+                name = %core.node.read().await.name(),
+                prefix = format_args!("({:b})", prefix),
+                age,
+                elder = is_elder,
                 cmd_id = %cmd_id,
+                section_key = ?section_key,
+                %command,
             )
         };
 
         async {
-            trace!("{:?} {}", LogMarker::CommandHandleStart, command);
-            trace!(?command);
+            trace!("{:?}", LogMarker::CommandHandleStart);
 
             let command_display = command.to_string();
             match self.try_handle_command(command).await {
@@ -187,7 +185,10 @@ impl Dispatcher {
                     Ok(outcome)
                 }
                 Err(error) => {
-                    error!("Error encountered when handling command: {:?}", error);
+                    error!(
+                        "Error encountered when handling command (cmd_id {}): {:?}",
+                        cmd_id, error
+                    );
                     trace!(
                         "{:?} {}: {:?}",
                         LogMarker::CommandHandleError,
@@ -204,41 +205,6 @@ impl Dispatcher {
 
     async fn try_handle_command(&self, command: Command) -> Result<Vec<Command>> {
         match command {
-            // Data node msg that requires no locking
-            Command::HandleNonBlockingMessage {
-                msg_id,
-                msg_authority,
-                dst_location,
-                msg,
-                sender,
-                known_keys,
-            } => {
-                self.core
-                    .read()
-                    .await
-                    .handle_non_blocking_message(
-                        msg_id,
-                        msg_authority,
-                        dst_location,
-                        msg,
-                        sender,
-                        known_keys,
-                    )
-                    .await
-            }
-            // Non-data node msg that requires locking
-            Command::HandleBlockingMessage {
-                sender,
-                msg_id,
-                msg_authority,
-                msg,
-            } => {
-                self.core
-                    .write()
-                    .await
-                    .handle_blocking_message(sender, msg_id, msg_authority, msg)
-                    .await
-            }
             Command::HandleSystemMessage {
                 sender,
                 msg_id,
@@ -249,8 +215,6 @@ impl Dispatcher {
                 known_keys,
             } => {
                 self.core
-                    .read()
-                    .await
                     .handle_system_message(
                         sender,
                         msg_id,
@@ -263,43 +227,41 @@ impl Dispatcher {
                     .await
             }
             Command::PrepareNodeMsgToSend { msg, dst } => {
-                self.core.read().await.prepare_node_msg(msg, dst)
+                self.core.prepare_node_msg(msg, dst).await
             }
             Command::HandleMessage {
-                sender,
+                sender_addr,
                 wire_msg,
                 original_bytes,
             } => {
                 self.core
-                    .read()
-                    .await
-                    .handle_message(sender, wire_msg, original_bytes)
+                    .handle_message(sender_addr, wire_msg, original_bytes)
                     .await
             }
-            Command::HandleTimeout(token) => self.core.write().await.handle_timeout(token),
+            Command::HandleTimeout(token) => self.core.handle_timeout(token).await,
             Command::HandleAgreement { proposal, sig } => {
-                self.core
-                    .write()
-                    .await
-                    .handle_agreement(proposal, sig)
-                    .await
+                self.core.handle_non_elder_agreement(proposal, sig).await
             }
-            Command::HandlePeerLost(addr) => self.core.read().await.handle_peer_lost(&addr).await,
+            Command::HandleElderAgreement { proposal, sig } => match proposal {
+                Proposal::OurElders(section_auth) => {
+                    self.core
+                        .handle_our_elders_agreement(section_auth.into_authed_state(), sig)
+                        .await
+                }
+                _ => {
+                    error!("Other agreement messages should be handled in `HandleAgreement`, which is non-blocking ");
+                    Ok(vec![])
+                }
+            },
+            Command::HandlePeerLost(peer) => self.core.handle_peer_lost(&peer.addr()).await,
             Command::HandleDkgOutcome {
                 section_auth,
                 outcome,
-            } => {
-                self.core
-                    .write()
-                    .await
-                    .handle_dkg_outcome(section_auth, outcome)
-                    .await
-            }
+            } => self.core.handle_dkg_outcome(section_auth, outcome).await,
             Command::HandleDkgFailure(signeds) => self
                 .core
-                .write()
-                .await
                 .handle_dkg_failure(signeds)
+                .await
                 .map(|command| vec![command]),
             Command::SendMessage {
                 recipients,
@@ -327,75 +289,56 @@ impl Dispatcher {
                 Ok(vec![])
             }
             Command::SetJoinsAllowed(joins_allowed) => {
-                self.core
-                    .read()
-                    .await
-                    .set_joins_allowed(joins_allowed)
-                    .await
+                self.core.set_joins_allowed(joins_allowed).await
             }
             Command::ProposeOnline {
-                mut peer,
+                peer,
                 previous_name,
                 dst_key,
             } => {
-                // The reachability check was completed during the initial bootstrap phase
-                peer.set_reachable(true);
                 self.core
-                    .read()
-                    .await
-                    .make_online_proposal(peer, previous_name, dst_key)
+                    .make_online_proposal(&peer, previous_name, dst_key)
                     .await
             }
-            Command::ProposeOffline(name) => self.core.read().await.propose_offline(name).await,
+            Command::ProposeOffline(name) => self.core.propose_offline(name).await,
             Command::StartConnectivityTest(name) => {
                 let msg = {
-                    let core = self.core.read().await;
-                    let node = core.node();
-                    let section_pk = *core.section().chain.last_key();
+                    let core = &self.core;
+                    let node = core.node.read().await.clone();
+                    let section_pk = core.network_knowledge().section_key().await;
                     WireMsg::single_src(
-                        node,
+                        &node,
                         DstLocation::Section {
-                            name: core.node().name(),
+                            name: node.name(),
                             section_pk,
                         },
                         SystemMsg::StartConnectivityTest(name),
                         section_pk,
                     )?
                 };
-                let our_name = self.core.read().await.node().name();
+                let our_name = self.core.node.read().await.name();
                 let peers = self
                     .core
-                    .read()
-                    .await
-                    .section()
+                    .network_knowledge()
                     .active_members()
+                    .await
                     .iter()
-                    .filter(|peer| peer.name() != &name && peer.name() != &our_name)
+                    .filter(|peer| peer.name() != name && peer.name() != our_name)
                     .cloned()
                     .collect_vec();
-                Ok(self.core.read().await.send_or_handle(msg, peers))
+                Ok(self.core.send_or_handle(msg, peers).await)
             }
             Command::TestConnectivity(name) => {
                 let mut commands = vec![];
-                if let Some(peer) = self
-                    .core
-                    .read()
-                    .await
-                    .section()
-                    .members()
-                    .get(&name)
-                    .map(|member_info| member_info.peer)
-                {
+                if let Some(member_info) = self.core.network_knowledge().members().get(&name) {
                     if self
                         .core
-                        .read()
-                        .await
                         .comm
-                        .is_reachable(peer.addr())
+                        .is_reachable(&member_info.addr())
                         .await
                         .is_err()
                     {
-                        commands.push(Command::ProposeOffline(*peer.name()));
+                        commands.push(Command::ProposeOffline(member_info.name()));
                     }
                 }
                 Ok(commands)
@@ -405,7 +348,7 @@ impl Dispatcher {
 
     async fn send_message(
         &self,
-        recipients: &[(XorName, SocketAddr)],
+        recipients: &[Peer],
         delivery_group_size: usize,
         wire_msg: WireMsg,
     ) -> Result<Vec<Command>> {
@@ -415,10 +358,8 @@ impl Dispatcher {
                     .await?
             }
             MsgKind::ServiceMsg(_) => {
-                let _ = self
+                let _res = self
                     .core
-                    .read()
-                    .await
                     .comm
                     .send_on_existing_connection(recipients, wire_msg)
                     .await;
@@ -432,14 +373,12 @@ impl Dispatcher {
 
     async fn deliver_messages(
         &self,
-        recipients: &[(XorName, SocketAddr)],
+        recipients: &[Peer],
         delivery_group_size: usize,
         wire_msg: WireMsg,
     ) -> Result<Vec<Command>> {
         let status = self
             .core
-            .read()
-            .await
             .comm
             .send(recipients, delivery_group_size, wire_msg)
             .await?;
@@ -458,13 +397,7 @@ impl Dispatcher {
     /// Send a message, either section to section, node to node, or to an end user.
     pub(super) async fn send_wire_message(&self, mut wire_msg: WireMsg) -> Result<Vec<Command>> {
         if let DstLocation::EndUser(EndUser(name)) = wire_msg.dst_location() {
-            let addr = self
-                .core
-                .read()
-                .await
-                .comm
-                .get_socket_addr_by_id(name)
-                .await;
+            let addr = self.core.comm.get_peer_address(name).await;
 
             if let Some(socket_addr) = addr {
                 // Send a message to a client peer.
@@ -472,9 +405,8 @@ impl Dispatcher {
                 // or validated as part of the routing library.
                 debug!("Sending client msg to {:?}: {:?}", socket_addr, wire_msg);
 
-                let recipients = vec![(*name, socket_addr)];
-                wire_msg
-                    .set_dst_section_pk(*self.core.read().await.section_chain().clone().last_key());
+                let recipients = vec![Peer::new(*name, socket_addr)];
+                wire_msg.set_dst_section_pk(self.core.network_knowledge().section_key().await);
 
                 let command = Command::SendMessage {
                     recipients,
@@ -491,7 +423,7 @@ impl Dispatcher {
             }
         } else {
             // This message is not for an end user, then send it to peer/s over the network
-            let cmd = self.core.read().await.send_msg_to_peers(wire_msg)?;
+            let cmd = self.core.send_msg_to_peers(wire_msg).await?;
             Ok(vec![cmd])
         }
     }
@@ -510,18 +442,22 @@ impl Dispatcher {
         }
     }
 
-    async fn handle_relocation_complete(&self, new_node: Node, new_section: Section) -> Result<()> {
-        let previous_name = self.core.read().await.node().name();
+    async fn handle_relocation_complete(
+        &self,
+        new_node: Node,
+        new_section: NetworkKnowledge,
+    ) -> Result<()> {
+        let previous_name = self.core.node.read().await.name();
         let new_keypair = new_node.keypair.clone();
 
-        let mut core = self.core.write().await;
-        *core = core.relocated(new_node, new_section).await?;
+        self.core.relocate(new_node, new_section).await?;
 
-        core.send_event(Event::Relocated {
-            previous_name,
-            new_keypair,
-        })
-        .await;
+        self.core
+            .send_event(Event::Relocated {
+                previous_name,
+                new_keypair,
+            })
+            .await;
 
         Ok(())
     }
