@@ -23,20 +23,16 @@ use self::{
     event::{Elders, Event, NodeElderChange},
     event_stream::EventStream,
 };
-use crate::messaging::{
-    data::StorageLevel,
-    system::{Peer, SystemMsg},
-    DstLocation, SectionAuthorityProvider, WireMsg,
-};
+use crate::messaging::{data::StorageLevel, system::SystemMsg, DstLocation, WireMsg};
 use crate::routing::{
     core::{join_network, ChunkStore, Comm, ConnectionEvent, Core, RegisterStorage},
     ed25519,
     error::{Error, Result},
     log_markers::LogMarker,
     messages::WireMsgUtils,
+    network_knowledge::SectionAuthorityProvider,
     node::Node,
-    peer::PeerUtils,
-    SectionAuthorityProviderUtils, MIN_ADULT_AGE,
+    Peer, MIN_ADULT_AGE,
 };
 use crate::{dbs::UsedSpace, messaging::data::ChunkDataExchange};
 use ed25519_dalek::{PublicKey, Signature, Signer, KEYPAIR_LENGTH};
@@ -99,16 +95,25 @@ impl Routing {
             )
             .await?;
             let node = Node::new(keypair, comm.our_connection_info());
-            let core = Core::first_node(comm, node, event_tx, used_space, root_storage_dir.clone())
-                .await?;
 
-            let section = core.section();
+            let genesis_sk_set = bls::SecretKeySet::random(0, &mut rand::thread_rng());
+            let core = Core::first_node(
+                comm,
+                node,
+                event_tx,
+                used_space,
+                root_storage_dir.clone(),
+                genesis_sk_set,
+            )
+            .await?;
+
+            let network_knowledge = core.network_knowledge();
 
             let elders = Elders {
-                prefix: *section.prefix(),
-                key: *section.chain().last_key(),
+                prefix: network_knowledge.prefix().await,
+                key: network_knowledge.section_key().await,
                 remaining: BTreeSet::new(),
-                added: section.authority_provider().names(),
+                added: network_knowledge.authority_provider().await.names(),
                 removed: BTreeSet::new(),
             };
 
@@ -118,10 +123,13 @@ impl Routing {
                 self_status_change: NodeElderChange::Promoted,
             })
             .await;
+
+            let genesis_key = network_knowledge.genesis_key();
             info!(
-                "{} Genesis node started!. Genesis key: {}",
+                "{} Genesis node started!. Genesis key {:?}, hex: {}",
                 node_name,
-                hex::encode(section.genesis_key().to_bytes())
+                genesis_key,
+                hex::encode(genesis_key.to_bytes())
             );
 
             core
@@ -163,8 +171,9 @@ impl Routing {
                 bootstrap_addr,
                 genesis_key
             );
+
             let joining_node = Node::new(keypair, comm.our_connection_info());
-            let (node, section) = join_network(
+            let (node, network_knowledge) = join_network(
                 joining_node,
                 &comm,
                 &mut connection_event_rx,
@@ -172,10 +181,11 @@ impl Routing {
                 genesis_key,
             )
             .await?;
+
             let core = Core::new(
                 comm,
                 node,
-                section,
+                network_knowledge,
                 None,
                 event_tx,
                 used_space,
@@ -183,7 +193,7 @@ impl Routing {
                 false,
             )
             .await?;
-            info!("{} Joined the network!", core.node().name());
+            info!("{} Joined the network!", core.node.read().await.name());
 
             core
         };
@@ -192,7 +202,7 @@ impl Routing {
         let event_stream = EventStream::new(event_rx);
 
         // Start listening to incoming connections.
-        let _ = task::spawn(handle_connection_events(
+        let _handle = task::spawn(handle_connection_events(
             dispatcher.clone(),
             connection_event_rx,
         ));
@@ -229,12 +239,7 @@ impl Routing {
     }
 
     pub(crate) async fn update_chunks(&self, chunks: ChunkDataExchange) {
-        self.dispatcher
-            .core
-            .read()
-            .await
-            .update_chunks(chunks)
-            .await
+        self.dispatcher.core.update_chunks(chunks).await
     }
 
     /// Sets the JoinsAllowed flag.
@@ -251,22 +256,22 @@ impl Routing {
 
     /// Returns the current age of this node.
     pub async fn age(&self) -> u8 {
-        self.dispatcher.core.read().await.node().age()
+        self.dispatcher.core.node.read().await.age()
     }
 
     /// Returns the ed25519 public key of this node.
     pub async fn public_key(&self) -> PublicKey {
-        self.dispatcher.core.read().await.node().keypair.public
+        self.dispatcher.core.node.read().await.keypair.public
     }
 
     /// Returns the ed25519 keypair of this node, as bytes.
     pub async fn keypair_as_bytes(&self) -> [u8; KEYPAIR_LENGTH] {
-        self.dispatcher.core.read().await.node().keypair.to_bytes()
+        self.dispatcher.core.node.read().await.keypair.to_bytes()
     }
 
     /// Signs `data` with the ed25519 key of this node.
     pub async fn sign_as_node(&self, data: &[u8]) -> Signature {
-        self.dispatcher.core.read().await.node().keypair.sign(data)
+        self.dispatcher.core.node.read().await.keypair.sign(data)
     }
 
     /// Signs `data` with the BLS secret key share of this node, if it has any. Returns
@@ -278,8 +283,6 @@ impl Routing {
     ) -> Result<(usize, bls::SignatureShare)> {
         self.dispatcher
             .core
-            .read()
-            .await
             .sign_with_section_key_share(data, public_key)
             .await
     }
@@ -288,9 +291,9 @@ impl Routing {
     pub async fn verify(&self, data: &[u8], signature: &Signature) -> bool {
         self.dispatcher
             .core
+            .node
             .read()
             .await
-            .node()
             .keypair
             .verify(data, signature)
             .is_ok()
@@ -298,27 +301,27 @@ impl Routing {
 
     /// The name of this node.
     pub async fn name(&self) -> XorName {
-        self.dispatcher.core.read().await.node().name()
+        self.dispatcher.core.node.read().await.name()
     }
 
     /// Returns connection info of this node.
     pub async fn our_connection_info(&self) -> SocketAddr {
-        self.dispatcher.core.read().await.our_connection_info()
+        self.dispatcher.core.our_connection_info()
     }
 
     /// Returns the Section Signed Chain
     pub async fn section_chain(&self) -> SecuredLinkedList {
-        self.dispatcher.core.read().await.section_chain().clone()
+        self.dispatcher.core.section_chain().await
     }
 
     /// Returns the Section Chain's genesis key
     pub async fn genesis_key(&self) -> bls::PublicKey {
-        self.dispatcher.core.read().await.section().genesis_key
+        *self.dispatcher.core.network_knowledge().genesis_key()
     }
 
     /// Prefix of our section
     pub async fn our_prefix(&self) -> Prefix {
-        *self.dispatcher.core.read().await.section().prefix()
+        self.dispatcher.core.network_knowledge().prefix().await
     }
 
     /// Finds out if the given XorName matches our prefix.
@@ -328,19 +331,17 @@ impl Routing {
 
     /// Returns whether the node is Elder.
     pub async fn is_elder(&self) -> bool {
-        self.dispatcher.core.read().await.is_elder()
+        self.dispatcher.core.is_elder().await
     }
 
     /// Returns the information of all the current section elders.
     pub async fn our_elders(&self) -> Vec<Peer> {
         self.dispatcher
             .core
-            .read()
-            .await
-            .section()
+            .network_knowledge()
             .authority_provider()
+            .await
             .peers()
-            .collect()
     }
 
     /// Returns the elders of our section sorted by their distance to `name` (closest first).
@@ -348,13 +349,13 @@ impl Routing {
         self.our_elders()
             .await
             .into_iter()
-            .sorted_by(|lhs, rhs| name.cmp_distance(lhs.name(), rhs.name()))
+            .sorted_by(|lhs, rhs| name.cmp_distance(&lhs.name(), &rhs.name()))
             .collect()
     }
 
     /// Returns the information of all the current section adults.
     pub async fn our_adults(&self) -> Vec<Peer> {
-        self.dispatcher.core.read().await.section().adults()
+        self.dispatcher.core.network_knowledge().adults().await
     }
 
     /// Returns the adults of our section sorted by their distance to `name` (closest first).
@@ -363,24 +364,22 @@ impl Routing {
         self.our_adults()
             .await
             .into_iter()
-            .sorted_by(|lhs, rhs| name.cmp_distance(lhs.name(), rhs.name()))
+            .sorted_by(|lhs, rhs| name.cmp_distance(&lhs.name(), &rhs.name()))
             .collect()
     }
 
     /// Returns our section's authority provider.
-    pub async fn our_section_auth(&self) -> SectionAuthorityProvider {
+    pub(crate) async fn our_section_auth(&self) -> SectionAuthorityProvider {
         self.dispatcher
             .core
-            .read()
-            .await
-            .section()
+            .network_knowledge()
             .authority_provider()
-            .clone()
+            .await
     }
 
     /// Returns the info about the section matching the name.
     pub async fn matching_section(&self, name: &XorName) -> Result<SectionAuthorityProvider> {
-        self.dispatcher.core.read().await.matching_section(name)
+        self.dispatcher.core.matching_section(name).await
     }
 
     /// Builds a WireMsg signed by this Node
@@ -391,7 +390,7 @@ impl Routing {
     ) -> Result<WireMsg> {
         let src_section_pk = *self.section_chain().await.last_key();
         WireMsg::single_src(
-            self.dispatcher.core.read().await.node(),
+            &self.dispatcher.core.node.read().await.clone(),
             dst,
             node_msg,
             src_section_pk,
@@ -408,14 +407,7 @@ impl Routing {
         let src_section_pk = *self.section_chain().await.last_key();
 
         WireMsg::for_dst_accumulation(
-            &self
-                .dispatcher
-                .core
-                .read()
-                .await
-                .key_share()
-                .await
-                .map_err(|err| err)?,
+            &self.dispatcher.core.key_share().await.map_err(|err| err)?,
             src,
             dst,
             node_msg,
@@ -440,13 +432,13 @@ impl Routing {
     /// Returns the current BLS public key set if this node has one, or
     /// `Error::MissingSecretKeyShare` otherwise.
     pub async fn public_key_set(&self) -> Result<bls::PublicKeySet> {
-        self.dispatcher.core.read().await.public_key_set().await
+        self.dispatcher.core.public_key_set().await
     }
 
     /// Returns our index in the current BLS group if this node is a member of one, or
     /// `Error::MissingSecretKeyShare` otherwise.
     pub async fn our_index(&self) -> Result<usize> {
-        self.dispatcher.core.read().await.our_index().await
+        self.dispatcher.core.our_index().await
     }
 }
 
@@ -457,11 +449,11 @@ async fn handle_connection_events(
 ) {
     while let Some(event) = incoming_conns.recv().await {
         match event {
-            ConnectionEvent::Received((sender, bytes)) => {
+            ConnectionEvent::Received((sender_addr, bytes)) => {
                 trace!(
                     "New message ({} bytes) received from: {}",
                     bytes.len(),
-                    sender
+                    sender_addr
                 );
 
                 // bytes.clone is cheap
@@ -474,24 +466,24 @@ async fn handle_connection_events(
                 };
 
                 let span = {
-                    let core = dispatcher.core.read().await;
-                    trace_span!("handle_message", name = %core.node().name(), %sender, msg_id = ?wire_msg.msg_id())
+                    let core = &dispatcher.core;
+                    trace_span!("handle_message", name = %core.node.read().await.name(), %sender_addr, msg_id = ?wire_msg.msg_id())
                 };
                 let _span_guard = span.enter();
                 let len = bytes.len();
                 let command = Command::HandleMessage {
-                    sender,
+                    sender_addr,
                     wire_msg,
                     original_bytes: Some(bytes),
                 };
                 trace!(
                     "{:?} from {} length {}",
                     LogMarker::DispatchHandleMsgCmd,
-                    sender,
+                    sender_addr,
                     len,
                 );
 
-                let _ = dispatcher.clone().handle_commands(command, None).await;
+                let _handle = dispatcher.clone().handle_commands(command, None).await;
             }
         }
     }
