@@ -14,14 +14,16 @@ pub mod register;
 /// Encoding utils
 pub mod utils;
 
+mod address;
 mod cache;
 mod chunk;
 mod errors;
 mod keys;
 mod token;
 
+pub use address::{BytesAddress, ChunkAddress, DataAddress, RegisterAddress, SafeKeyAddress};
 pub use cache::Cache;
-pub use chunk::{Address as ChunkAddress, Chunk, MAX_CHUNK_SIZE_IN_BYTES};
+pub use chunk::{Chunk, MAX_CHUNK_SIZE_IN_BYTES};
 pub use errors::{convert_dt_error_to_error_message, Error, Result};
 pub use keys::{
     keypair::{BlsKeypairShare, Encryption, Keypair, OwnerType, Signing},
@@ -30,17 +32,6 @@ pub use keys::{
     secret_key::SecretKey,
     signature::{Signature, SignatureShare},
 };
-pub use register::Address as RegisterAddress;
 pub use token::Token;
 
-use serde::{Deserialize, Serialize};
 use xor_name::XorName;
-
-/// Object storing an address of data on the network
-#[derive(Clone, Eq, PartialEq, Hash, Serialize, Deserialize, Debug, PartialOrd, Ord)]
-pub enum DataAddress {
-    /// Chunk Address
-    Chunk(ChunkAddress),
-    /// Register Address
-    Register(RegisterAddress),
-}
